@@ -1,4 +1,3 @@
-/* $Xorg: security.c,v 1.4 2001/02/09 02:04:32 xorgcvs Exp $ */
 /*
 
 Copyright 1996, 1998  The Open Group
@@ -24,7 +23,53 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.17 2003/12/04 16:59:35 tsi Exp $ */
+/*
+ * Copyright (c) 1996-2004 by The XFree86 Project, Inc.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject
+ * to the following conditions:
+ *
+ *   1.  Redistributions of source code must retain the above copyright
+ *       notice, this list of conditions, and the following disclaimer.
+ *
+ *   2.  Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer
+ *       in the documentation and/or other materials provided with the
+ *       distribution, and in the same place and form as other copyright,
+ *       license and disclaimer information.
+ *
+ *   3.  The end-user documentation included with the redistribution,
+ *       if any, must include the following acknowledgment: "This product
+ *       includes software developed by The XFree86 Project, Inc
+ *       (http://www.xfree86.org/) and its contributors", in the same
+ *       place and form as other third-party acknowledgments.  Alternately,
+ *       this acknowledgment may appear in the software itself, in the
+ *       same form and location as other such third-party acknowledgments.
+ *
+ *   4.  Except as contained in this notice, the name of The XFree86
+ *       Project, Inc shall not be used in advertising or otherwise to
+ *       promote the sale, use or other dealings in this Software without
+ *       prior written authorization from The XFree86 Project, Inc.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE XFREE86 PROJECT, INC OR ITS CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.20 2005/10/14 15:16:11 tsi Exp $ */
 
 #include "dixstruct.h"
 #include "extnsionst.h"
@@ -35,19 +80,19 @@ in this Software without prior written authorization from The Open Group.
 #include "colormapst.h"
 #include "propertyst.h"
 #define _SECURITY_SERVER
-#include "securstr.h"
+#include <X11/extensions/securstr.h>
 #include <assert.h>
 #include <stdarg.h>
 #ifdef LBX
 #define _XLBX_SERVER_
-#include "XLbx.h"
+#include <X11/extensions/XLbx.h>
 extern unsigned char LbxReqCode;
 #endif
 #ifdef XAPPGROUP
-#include "Xagsrv.h"
+#include <X11/extensions/Xagsrv.h>
 #endif
 #include <stdio.h>  /* for file reading operations */
-#include "Xatom.h"  /* for XA_STRING */
+#include <X11/Xatom.h>  /* for XA_STRING */
 
 #ifndef DEFAULTPOLICYFILE
 # define DEFAULTPOLICYFILE NULL
@@ -59,6 +104,8 @@ extern unsigned char LbxReqCode;
 
 #include "modinit.h"
 
+#ifdef XCSECURITY
+
 static int SecurityErrorBase;  /* first Security error number */
 static int SecurityEventBase;  /* first Security event number */
 
@@ -121,9 +168,7 @@ SecurityAudit(char *format, ...)
  */
 
 static int
-SecurityDeleteAuthorization(
-    pointer value,
-    XID id)
+SecurityDeleteAuthorization(pointer value, XID id)
 {
     SecurityAuthorizationPtr pAuth = (SecurityAuthorizationPtr)value;
     unsigned short name_len, data_len;
@@ -180,9 +225,7 @@ SecurityDeleteAuthorization(
 
 /* resource delete function for RTEventClient */
 static int
-SecurityDeleteAuthorizationEventClient(
-    pointer value,
-    XID id)
+SecurityDeleteAuthorizationEventClient(pointer value, XID id)
 {
     OtherClientsPtr pEventClient, prev = NULL;
     SecurityAuthorizationPtr pAuth = (SecurityAuthorizationPtr)value;
@@ -222,9 +265,8 @@ SecurityDeleteAuthorizationEventClient(
  */
 
 static CARD32
-SecurityComputeAuthorizationTimeout(
-    SecurityAuthorizationPtr pAuth,
-    unsigned int seconds)
+SecurityComputeAuthorizationTimeout(SecurityAuthorizationPtr pAuth,
+				    unsigned int seconds)
 {
     /* maxSecs is the number of full seconds that can be expressed in
      * 32 bits worth of milliseconds
@@ -263,10 +305,7 @@ SecurityComputeAuthorizationTimeout(
  */
 
 static CARD32
-SecurityAuthorizationExpired(
-    OsTimerPtr timer,
-    CARD32 time,
-    pointer pval)
+SecurityAuthorizationExpired(OsTimerPtr timer, CARD32 time, pointer pval)
 {
     SecurityAuthorizationPtr pAuth = (SecurityAuthorizationPtr)pval;
 
@@ -298,8 +337,7 @@ SecurityAuthorizationExpired(
  */
 
 static void
-SecurityStartAuthorizationTimer(
-    SecurityAuthorizationPtr pAuth)
+SecurityStartAuthorizationTimer(SecurityAuthorizationPtr pAuth)
 {
     pAuth->timer = TimerSet(pAuth->timer, 0,
 	SecurityComputeAuthorizationTimeout(pAuth, pAuth->timeout),
@@ -312,8 +350,7 @@ SecurityStartAuthorizationTimer(
  */
 
 static int
-ProcSecurityQueryVersion(
-    ClientPtr client)
+ProcSecurityQueryVersion(ClientPtr client)
 {
     /* REQUEST(xSecurityQueryVersionReq); */
     xSecurityQueryVersionReply 	rep;
@@ -332,7 +369,7 @@ ProcSecurityQueryVersion(
     rep.minorVersion  	= SECURITY_MINOR_VERSION;
     if(client->swapped)
     {
-	register char n;
+	char n;
     	swaps(&rep.sequenceNumber, n);
 	swaps(&rep.majorVersion, n);
 	swaps(&rep.minorVersion, n);
@@ -344,10 +381,8 @@ ProcSecurityQueryVersion(
 
 
 static int
-SecurityEventSelectForAuthorization(
-    SecurityAuthorizationPtr pAuth,
-    ClientPtr client,
-    Mask mask)
+SecurityEventSelectForAuthorization(SecurityAuthorizationPtr pAuth,
+				    ClientPtr client, Mask mask)
 {
     OtherClients *pEventClient;
 
@@ -384,8 +419,7 @@ SecurityEventSelectForAuthorization(
 
 
 static int
-ProcSecurityGenerateAuthorization(
-    ClientPtr client)
+ProcSecurityGenerateAuthorization(ClientPtr client)
 {
     REQUEST(xSecurityGenerateAuthorizationReq);
     int len;			/* request length in CARD32s*/
@@ -550,7 +584,7 @@ ProcSecurityGenerateAuthorization(
 
     if (client->swapped)
     {
-	register char n;
+	char n;
     	swapl(&rep.length, n);
     	swaps(&rep.sequenceNumber, n);
     	swapl(&rep.authId, n);
@@ -581,8 +615,7 @@ bailout:
 } /* ProcSecurityGenerateAuthorization */
 
 static int
-ProcSecurityRevokeAuthorization(
-    ClientPtr client)
+ProcSecurityRevokeAuthorization(ClientPtr client)
 {
     REQUEST(xSecurityRevokeAuthorizationReq);
     SecurityAuthorizationPtr pAuth;
@@ -606,8 +639,7 @@ ProcSecurityRevokeAuthorization(
 
 
 static int
-ProcSecurityDispatch(
-    ClientPtr client)
+ProcSecurityDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
@@ -625,11 +657,10 @@ ProcSecurityDispatch(
 } /* ProcSecurityDispatch */
 
 static int
-SProcSecurityQueryVersion(
-    ClientPtr client)
+SProcSecurityQueryVersion(ClientPtr client)
 {
     REQUEST(xSecurityQueryVersionReq);
-    register char 	n;
+    char 	n;
 
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xSecurityQueryVersionReq);
@@ -640,11 +671,10 @@ SProcSecurityQueryVersion(
 
 
 static int
-SProcSecurityGenerateAuthorization(
-    ClientPtr client)
+SProcSecurityGenerateAuthorization(ClientPtr client)
 {
     REQUEST(xSecurityGenerateAuthorizationReq);
-    register char 	n;
+    char 	n;
     CARD32 *values;
     unsigned long nvalues;
 
@@ -663,11 +693,10 @@ SProcSecurityGenerateAuthorization(
 
 
 static int
-SProcSecurityRevokeAuthorization(
-    ClientPtr client)
+SProcSecurityRevokeAuthorization(ClientPtr client)
 {
     REQUEST(xSecurityRevokeAuthorizationReq);
-    register char 	n;
+    char 	n;
 
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xSecurityRevokeAuthorizationReq);
@@ -677,8 +706,7 @@ SProcSecurityRevokeAuthorization(
 
 
 static int
-SProcSecurityDispatch(
-    ClientPtr client)
+SProcSecurityDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
@@ -696,9 +724,8 @@ SProcSecurityDispatch(
 } /* SProcSecurityDispatch */
 
 static void 
-SwapSecurityAuthorizationRevokedEvent(
-    xSecurityAuthorizationRevokedEvent *from,
-    xSecurityAuthorizationRevokedEvent *to)
+SwapSecurityAuthorizationRevokedEvent(xSecurityAuthorizationRevokedEvent *from,
+				      xSecurityAuthorizationRevokedEvent *to)
 {
     to->type = from->type;
     to->detail = from->detail;
@@ -726,10 +753,8 @@ SwapSecurityAuthorizationRevokedEvent(
  */
 
 static void
-SecurityDetermineEventPropogationLimits(
-    DeviceIntPtr dev,
-    WindowPtr *ppWin,
-    WindowPtr *ppStopWin)
+SecurityDetermineEventPropogationLimits(DeviceIntPtr dev, WindowPtr *ppWin,
+					WindowPtr *ppStopWin)
 {
     WindowPtr pFocusWin = dev->focus ? dev->focus->win : NoneWin;
 
@@ -776,10 +801,7 @@ SecurityDetermineEventPropogationLimits(
  */
 
 Bool
-SecurityCheckDeviceAccess(client, dev, fromRequest)
-    ClientPtr client;
-    DeviceIntPtr dev;
-    Bool fromRequest;
+SecurityCheckDeviceAccess(ClientPtr client, DeviceIntPtr dev, Bool fromRequest)
 {
     WindowPtr pWin, pStopWin;
     Bool untrusted_got_event;
@@ -896,9 +918,7 @@ SecurityCheckDeviceAccess(client, dev, fromRequest)
  */
 
 static pointer
-SecurityAuditResourceIDAccess(
-    ClientPtr client,
-    XID id)
+SecurityAuditResourceIDAccess(ClientPtr client, XID id)
 {
     int cid = CLIENT_ID(id);
     int reqtype = ((xReq *)client->requestBuffer)->reqType;
@@ -950,12 +970,8 @@ SecurityAuditResourceIDAccess(
  */
 
 static pointer
-SecurityCheckResourceIDAccess(
-    ClientPtr client,
-    XID id,
-    RESTYPE rtype,
-    Mask access_mode,
-    pointer rval)
+SecurityCheckResourceIDAccess(ClientPtr client, XID id, RESTYPE rtype,
+			      Mask access_mode, pointer rval)
 {
     int cid = CLIENT_ID(id);
     int reqtype = ((xReq *)client->requestBuffer)->reqType;
@@ -1116,10 +1132,8 @@ SecurityCheckResourceIDAccess(
  */
 
 static void
-SecurityClientStateCallback(
-    CallbackListPtr *pcbl,
-    pointer nulldata,
-    pointer calldata)
+SecurityClientStateCallback(CallbackListPtr *pcbl, pointer nulldata,
+			    pointer calldata)
 {
     NewClientInfoRec *pci = (NewClientInfoRec *)calldata;
     ClientPtr client = pci->client;
@@ -1175,9 +1189,7 @@ SecurityClientStateCallback(
 
 #ifdef LBX
 Bool
-SecuritySameLevel(client, authId)
-    ClientPtr client;
-    XID authId;
+SecuritySameLevel(ClientPtr client, XID authId)
 {
     SecurityAuthorizationPtr pAuth;
 
@@ -1210,15 +1222,9 @@ SecuritySameLevel(client, authId)
  *	region of the window will be destroyed (overwritten) in pBuf.
  */
 void
-SecurityCensorImage(client, pVisibleRegion, widthBytesLine, pDraw, x, y, w, h,
-		    format, pBuf)
-    ClientPtr client;
-    RegionPtr pVisibleRegion;
-    long widthBytesLine;
-    DrawablePtr pDraw;
-    int x, y, w, h;
-    unsigned int format;
-    char * pBuf;
+SecurityCensorImage(ClientPtr client, RegionPtr pVisibleRegion,
+		    long widthBytesLine, DrawablePtr pDraw, int x, int y,
+		    int w, int h, unsigned int format, char *pBuf)
 {
     ScreenPtr pScreen = pDraw->pScreen;
     RegionRec imageRegion;  /* region representing x,y,w,h */
@@ -1366,8 +1372,7 @@ SecurityFreePropertyAccessList(void)
 #endif
 
 static char *
-SecuritySkipWhitespace(
-    char *p)
+SecuritySkipWhitespace(char *p)
 {
     while (SecurityIsWhitespace(*p))
 	p++;
@@ -1376,8 +1381,7 @@ SecuritySkipWhitespace(
 
 
 static char *
-SecurityParseString(
-    char **rest)
+SecurityParseString(char **rest)
 {
     char *startOfString;
     char *s = *rest;
@@ -1413,8 +1417,7 @@ SecurityParseString(
 
 
 static int
-SecurityParseKeyword(
-    char **p)
+SecurityParseKeyword(char **p)
 {
     int i;
     char *s = *p;
@@ -1434,8 +1437,7 @@ SecurityParseKeyword(
 
 
 static Bool
-SecurityParsePropertyAccessRule(
-    char *p)
+SecurityParsePropertyAccessRule(char *p)
 {
     char *propname;
     char c;
@@ -1575,8 +1577,7 @@ static char **SecurityPolicyStrings = NULL;
 static int nSecurityPolicyStrings = 0;
 
 static Bool
-SecurityParseSitePolicy(
-    char *p)
+SecurityParseSitePolicy(char *p)
 {
     char *policyStr = SecurityParseString(&p);
     char *copyPolicyStr;
@@ -1606,8 +1607,7 @@ SecurityParseSitePolicy(
 
 
 char **
-SecurityGetSitePolicyStrings(n)
-    int *n;
+SecurityGetSitePolicyStrings(int *n)
 {
     *n = nSecurityPolicyStrings;
     return SecurityPolicyStrings;
@@ -1733,9 +1733,7 @@ SecurityLoadPropertyAccessList(void)
 
 
 static Bool
-SecurityMatchString(
-    char *ws,
-    char *cs)
+SecurityMatchString(char *ws, char *cs)
 {
     while (*ws && *cs)
     {
@@ -1767,11 +1765,8 @@ SecurityMatchString(
 
 
 char
-SecurityCheckPropertyAccess(client, pWin, propertyName, access_mode)
-    ClientPtr client;
-    WindowPtr pWin;
-    ATOM propertyName;
-    Mask access_mode;
+SecurityCheckPropertyAccess(ClientPtr client, WindowPtr pWin,
+			    ATOM propertyName, Mask access_mode)
 {
     PropertyAccessPtr pacl;
     char action = SecurityDefaultAction;
@@ -1909,8 +1904,7 @@ SecurityCheckPropertyAccess(client, pWin, propertyName, access_mode)
  */
 
 static void
-SecurityResetProc(
-    ExtensionEntry *extEntry)
+SecurityResetProc(ExtensionEntry *extEntry)
 {
     SecurityFreePropertyAccessList();
     SecurityFreeSitePolicyStrings();
@@ -1918,10 +1912,7 @@ SecurityResetProc(
 
 
 int
-XSecurityOptions(argc, argv, i)
-    int argc;
-    char **argv;
-    int i;
+XSecurityOptions(int argc, char **argv, int i)
 {
     if (strcmp(argv[i], "-sp") == 0)
     {
@@ -1997,3 +1988,5 @@ SecurityExtensionInit(INITARGS)
     SecurityLoadPropertyAccessList();
 
 } /* SecurityExtensionInit */
+
+#endif
