@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/Xext/xf86misc.c,v 3.42 2004/02/13 23:58:30 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/Xext/xf86misc.c,v 3.48 2005/10/14 15:16:12 tsi Exp $ */
 
 /*
  * Copyright (c) 1995, 1996  The XFree86 Project, Inc
@@ -51,8 +51,8 @@
 
 #define NEED_REPLIES
 #define NEED_EVENTS
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "misc.h"
 #include "dixstruct.h"
 #include "extnsionst.h"
@@ -61,10 +61,10 @@
 #include "servermd.h"
 #define _XF86MISC_SERVER_
 #undef _XF86MISC_SAVER_COMPAT_
-#include "xf86mscstr.h"
+#include <X11/extensions/xf86mscstr.h>
 #include "swaprep.h"
 #include "xf86.h"
-#include "Xfuncproto.h"
+#include <X11/Xfuncproto.h>
 #include "xf86miscproc.h"
 
 #if 0
@@ -202,17 +202,15 @@ XFree86MiscExtensionInit(void)
 
 /*ARGSUSED*/
 static void
-XF86MiscResetProc (extEntry)
-    ExtensionEntry* extEntry;
+XF86MiscResetProc(ExtensionEntry *extEntry)
 {
 }
 
 static int
-ProcXF86MiscQueryVersion(client)
-    register ClientPtr client;
+ProcXF86MiscQueryVersion(ClientPtr client)
 {
     xXF86MiscQueryVersionReply rep;
-    register int n;
+    int n;
 
     DEBUG_P("XF86MiscQueryVersion");
 
@@ -240,8 +238,7 @@ ProcXF86MiscQueryVersion(client)
  * clients.
  */
 static int
-ProcXF86MiscSetSaver(client)
-    register ClientPtr client;
+ProcXF86MiscSetSaver(ClientPtr client)
 {
     REQUEST(xXF86MiscSetSaverReq);
     ScrnInfoPtr vptr;
@@ -266,12 +263,11 @@ ProcXF86MiscSetSaver(client)
  * clients.
  */
 static int
-ProcXF86MiscGetSaver(client)
-    register ClientPtr client;
+ProcXF86MiscGetSaver(ClientPtr client)
 {
     REQUEST(xXF86MiscGetSaverReq);
     xXF86MiscGetSaverReply rep;
-    register int n;
+    int n;
     ScrnInfoPtr vptr;
 
     if (stuff->screen > screenInfo.numScreens)
@@ -299,13 +295,12 @@ ProcXF86MiscGetSaver(client)
 #endif /* _XF86MISC_SAVER_COMPAT_ */
 
 static int
-ProcXF86MiscGetMouseSettings(client)
-    register ClientPtr client;
+ProcXF86MiscGetMouseSettings(ClientPtr client)
 {
     xXF86MiscGetMouseSettingsReply rep;
     char *devname;
     pointer mouse;
-    register int n;
+    int n;
 
     DEBUG_P("XF86MiscGetMouseSettings");
 
@@ -351,12 +346,11 @@ ProcXF86MiscGetMouseSettings(client)
 }
 
 static int
-ProcXF86MiscGetKbdSettings(client)
-    register ClientPtr client;
+ProcXF86MiscGetKbdSettings(ClientPtr client)
 {
     xXF86MiscGetKbdSettingsReply rep;
     pointer kbd;
-    register int n;
+    int n;
 
     DEBUG_P("XF86MiscGetKbdSettings");
 
@@ -383,8 +377,7 @@ ProcXF86MiscGetKbdSettings(client)
 }
 
 static int
-ProcXF86MiscSetMouseSettings(client)
-    register ClientPtr client;
+ProcXF86MiscSetMouseSettings(ClientPtr client)
 {
     MiscExtReturn ret;
     pointer mouse;
@@ -460,8 +453,7 @@ ProcXF86MiscSetMouseSettings(client)
 }
 
 static int
-ProcXF86MiscSetKbdSettings(client)
-    register ClientPtr client;
+ProcXF86MiscSetKbdSettings(ClientPtr client)
 {
     MiscExtReturn ret;
     pointer kbd;
@@ -499,8 +491,7 @@ ProcXF86MiscSetKbdSettings(client)
 }
 
 static int
-ProcXF86MiscSetGrabKeysState(client)
-    register ClientPtr client;
+ProcXF86MiscSetGrabKeysState(ClientPtr client)
 {
     int n, status;
     xXF86MiscSetGrabKeysStateReply rep;
@@ -547,7 +538,8 @@ ProcXF86MiscSetClientVersion(ClientPtr client)
 	    return BadAlloc;
 	MPRIV(client) = pPriv;
     }
-    ErrorF("SetClientVersion: %i %i\n",stuff->major,stuff->minor);
+    xf86MsgVerb(X_INFO, 4, "SetClientVersion: %i %i\n",
+	stuff->major,stuff->minor);
     pPriv->major = stuff->major;
     pPriv->minor = stuff->minor;
     
@@ -555,14 +547,13 @@ ProcXF86MiscSetClientVersion(ClientPtr client)
 }
 
 static int
-ProcXF86MiscGetFilePaths(client)
-    register ClientPtr client;
+ProcXF86MiscGetFilePaths(ClientPtr client)
 {
     xXF86MiscGetFilePathsReply rep;
     const char *configfile;
     const char *modulepath;
     const char *logfile;
-    register int n;
+    int n;
 
     DEBUG_P("XF86MiscGetFilePaths");
 
@@ -601,13 +592,12 @@ ProcXF86MiscGetFilePaths(client)
 }
 
 static int
-ProcXF86MiscPassMessage(client)
-    register ClientPtr client;
+ProcXF86MiscPassMessage(ClientPtr client)
 {
     xXF86MiscPassMessageReply rep;
     char *msgtype, *msgval, *retstr;
     int retval, size;
-    register int n;
+    int n;
 
     REQUEST(xXF86MiscPassMessageReq);
 
@@ -625,20 +615,29 @@ ProcXF86MiscPassMessage(client)
 	strncpy(msgtype,(char*)(&stuff[1]),stuff->typelen);
     } else return BadValue;
     if (stuff->vallen) {
-	if (!(msgval = xalloc(stuff->vallen)))
+	if (!(msgval = xalloc(stuff->vallen))) {
+	    xfree(msgtype);
 	    return BadAlloc;
-	strncpy(msgval,(char*)(&stuff[1] + ((stuff->typelen + 3) & ~3)),
+	}
+	strncpy(msgval,(char*)((char*)&stuff[1] + ((stuff->typelen + 3) & ~3)),
 			stuff->vallen);
-    } else return BadValue;
+    } else {
+	xfree(msgtype);
+	return BadValue;
+    }
 
-    if ((retval= MiscExtPassMessage(stuff->screen,msgtype,msgval,&retstr)) != 0)
+    if ((retval = MiscExtPassMessage(stuff->screen,msgtype,msgval,&retstr)) != 0) {
+	xfree(msgtype);
+	xfree(msgval);
 	return retval;
+    }
 
     rep.type = X_Reply;
     rep.sequenceNumber = client->sequence;
-    rep.mesglen = (retstr? strlen(retstr): 0);
+    rep.mesglen = (retstr ? strlen(retstr) : 0);
     rep.length = (SIZEOF(xXF86MiscPassMessageReply) - SIZEOF(xGenericReply) +
 		  ((rep.mesglen + 3) & ~3)) >> 2;
+    rep.status = 0;
     
     if (client->swapped) {
     	swaps(&rep.sequenceNumber, n);
@@ -650,12 +649,13 @@ ProcXF86MiscPassMessage(client)
     if (rep.mesglen)
         WriteToClient(client, rep.mesglen, (char *)retstr);
 
+    xfree(msgtype);
+    xfree(msgval);
     return (client->noClientException);
 }
 
 static int
-ProcXF86MiscDispatch (client)
-    register ClientPtr	client;
+ProcXF86MiscDispatch(ClientPtr client)
 {
     REQUEST(xReq);
     switch (stuff->data)
@@ -698,10 +698,9 @@ ProcXF86MiscDispatch (client)
 }
 
 static int
-SProcXF86MiscQueryVersion(client)
-    register ClientPtr	client;
+SProcXF86MiscQueryVersion(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscQueryVersionReq);
     swaps(&stuff->length, n);
     return ProcXF86MiscQueryVersion(client);
@@ -709,10 +708,9 @@ SProcXF86MiscQueryVersion(client)
 
 #ifdef _XF86MISC_SAVER_COMPAT_
 static int
-SProcXF86MiscGetSaver(client)
-    ClientPtr client;
+SProcXF86MiscGetSaver(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscGetSaverReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscGetSaverReq);
@@ -721,10 +719,9 @@ SProcXF86MiscGetSaver(client)
 }
 
 static int
-SProcXF86MiscSetSaver(client)
-    ClientPtr client;
+SProcXF86MiscSetSaver(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscSetSaverReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscSetSaverReq);
@@ -736,10 +733,9 @@ SProcXF86MiscSetSaver(client)
 #endif /* _XF86MISC_SAVER_COMPAT_ */
 
 static int
-SProcXF86MiscGetMouseSettings(client)
-    ClientPtr client;
+SProcXF86MiscGetMouseSettings(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscGetMouseSettingsReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscGetMouseSettingsReq);
@@ -747,10 +743,9 @@ SProcXF86MiscGetMouseSettings(client)
 }
 
 static int
-SProcXF86MiscGetKbdSettings(client)
-    ClientPtr client;
+SProcXF86MiscGetKbdSettings(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscGetKbdSettingsReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscGetKbdSettingsReq);
@@ -758,10 +753,9 @@ SProcXF86MiscGetKbdSettings(client)
 }
 
 static int
-SProcXF86MiscSetMouseSettings(client)
-    ClientPtr client;
+SProcXF86MiscSetMouseSettings(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscSetMouseSettingsReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscSetMouseSettingsReq);
@@ -776,10 +770,9 @@ SProcXF86MiscSetMouseSettings(client)
 }
 
 static int
-SProcXF86MiscSetKbdSettings(client)
-    ClientPtr client;
+SProcXF86MiscSetKbdSettings(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscSetKbdSettingsReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscSetKbdSettingsReq);
@@ -790,10 +783,9 @@ SProcXF86MiscSetKbdSettings(client)
 }
 
 static int
-SProcXF86MiscSetGrabKeysState(client)
-    ClientPtr client;
+SProcXF86MiscSetGrabKeysState(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscSetGrabKeysStateReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscSetGrabKeysStateReq);
@@ -804,7 +796,7 @@ SProcXF86MiscSetGrabKeysState(client)
 static int
 SProcXF86MiscSetClientVersion(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscSetClientVersionReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscSetClientVersionReq);
@@ -814,10 +806,9 @@ SProcXF86MiscSetClientVersion(ClientPtr client)
 }
 
 static int
-SProcXF86MiscGetFilePaths(client)
-    ClientPtr client;
+SProcXF86MiscGetFilePaths(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscGetFilePathsReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscGetFilePathsReq);
@@ -825,10 +816,9 @@ SProcXF86MiscGetFilePaths(client)
 }
 
 static int
-SProcXF86MiscPassMessage(client)
-    ClientPtr client;
+SProcXF86MiscPassMessage(ClientPtr client)
 {
-    register int n;
+    int n;
     REQUEST(xXF86MiscPassMessageReq);
     swaps(&stuff->length, n);
     REQUEST_SIZE_MATCH(xXF86MiscPassMessageReq);
@@ -836,8 +826,7 @@ SProcXF86MiscPassMessage(client)
 }
 
 static int
-SProcXF86MiscDispatch (client)
-    register ClientPtr	client;
+SProcXF86MiscDispatch(ClientPtr client)
 {
     REQUEST(xReq);
     switch (stuff->data)
