@@ -1,5 +1,5 @@
 /*
- * $XFree86: xc/programs/Xserver/hw/tinyx/linux/mouse.c,v 1.1 2004/06/02 22:43:02 dawes Exp $
+ * $XFree86: xc/programs/Xserver/hw/tinyx/linux/mouse.c,v 1.2 2005/10/14 15:16:28 tsi Exp $
  *
  * Copyright  2001 Keith Packard, member of The XFree86 Project, Inc.
  *
@@ -69,12 +69,12 @@
  */
 
 #define NEED_EVENTS
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "inputstr.h"
 #include "scrnintstr.h"
 #include "tinyx.h"
-#include "Xpoll.h"
+#include <X11/Xpoll.h>
 #include <errno.h>
 #include <termios.h>
 
