@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Events.c,v 3.166 2005/02/18 22:38:31 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Events.c,v 3.174 2006/04/08 17:53:39 dawes Exp $ */
 /*
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  *
@@ -22,7 +22,7 @@
  *
  */
 /*
- * Copyright (c) 1994-2003 by The XFree86 Project, Inc.
+ * Copyright (c) 1994-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -68,19 +68,21 @@
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* $XConsortium: xf86Events.c /main/46 1996/10/25 11:36:30 kaleb $ */
-
 /* [JCH-96/01/21] Extended std reverse map to four buttons. */
 
 #ifdef __UNIXOS2__
 #define I_NEED_OS2_H
 #endif
 
-#include "X.h"
-#include "Xpoll.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xpoll.h>
+#include <X11/Xproto.h>
 #include "misc.h"
 
+#ifdef XFree86LOADER
+#include "loaderProcs.h"
+#endif
+
 #include "compiler.h"
 
 #include "xf86.h"
@@ -95,8 +97,8 @@
 #endif
 
 #ifdef XINPUT
-#include "XI.h"
-#include "XIproto.h"
+#include <X11/extensions/XI.h>
+#include <X11/extensions/XIproto.h>
 #else
 #include "inputstr.h"
 #endif
@@ -107,11 +109,15 @@
 
 #ifdef XF86BIGFONT
 #define _XF86BIGFONT_SERVER_
-#include "xf86bigfont.h"
+#include <X11/extensions/xf86bigfont.h>
+#endif
+
+#ifdef STACKTRACE
+#include "getstack.h"
 #endif
 
 #ifdef XKB
-#include "XKBsrv.h"
+#include <X11/extensions/XKBsrv.h>
 #endif
 
 #define XE_POINTER  1
@@ -132,7 +138,7 @@
 #ifdef XTESTEXT1
 
 #define	XTestSERVER_SIDE
-#include "xtestext1.h"
+#include <X11/extensions/xtestext1.h>
 extern short xtest_mousex;
 extern short xtest_mousey;
 extern int   on_steal_input;          
@@ -317,9 +323,6 @@ xf86ProcessActionEvent(ActionEvent action, void *arg)
     switch (action) {
     case ACTION_TERMINATE:
 	if (!xf86Info.dontZap) {
-#ifdef XFreeXDGA
-	    DGAShutdown();
-#endif
 	    GiveUp(0);
 	}
 	break;
@@ -1270,10 +1273,44 @@ xf86RemoveEnabledDevice(InputInfoPtr pInfo)
     }
 }
 
-static int *xf86SignalIntercept = NULL;
+#ifdef STACKTRACE
+
+#ifndef STACK_LEVELS
+#define STACK_LEVELS 16
+#endif
+void
+xf86ShowStackTrace()
+{
+    unsigned long returnStack[STACK_LEVELS];
+    int i;
+
+    getStackTrace(returnStack, STACK_LEVELS);
+    ErrorF("Stack trace:\n");
+    for (i = 0; i < STACK_LEVELS && returnStack[i]; i++) {
+      ErrorF("%2d: 0x%lx", i, returnStack[i]);
+#ifdef XFree86LOADER
+      ErrorF(": ");
+      if (!LoaderPrintSymbol(returnStack[i]))
+	break;
+#else
+      ErrorF("\n");
+#endif
+    }
+}
+
+#else
 
 void
-xf86InterceptSignals(int *signo)
+xf86ShowStackTrace()
+{
+    ErrorF("Stack trace not supported on this platform.\n");
+}
+#endif
+
+static volatile int *xf86SignalIntercept = NULL;
+
+void
+xf86InterceptSignals(volatile int *signo)
 {
     if ((xf86SignalIntercept = signo))
 	*signo = -1;
@@ -1300,12 +1337,16 @@ xf86SigHandler(int signo)
 #endif
 #if defined(XFree86LOADER)
   if (xf86Initialising)
-      LoaderCheckUnresolved(LD_RESOLV_IFDONE);
+      LoaderCheckUnresolved(0);
   ErrorF("\n"
 	 "   *** If unresolved symbols were reported above, they might not\n"
-	 "   *** be the reason for the server aborting.\n");
+	 "   *** be the reason for the server aborting.\n\n");
 #endif
-  FatalError("Caught signal %d.  Server aborting\n", signo);
+  ErrorF("Caught signal %d.\n", signo);
+
+  xf86ShowStackTrace();
+
+  FatalError("Server aborting\n");
 }
 
 #ifdef MEMDEBUG
@@ -1623,8 +1664,6 @@ XTestGenerateEvent(int dev_type, int keycode, int keystate, int mousex,
 
 /* XXX Currently XKB is mandatory. */
 
-extern int WSKbdToKeycode(int);
-
 void
 xf86PostWSKbdEvent(struct wscons_event *event)
 {
@@ -1638,11 +1677,13 @@ xf86PostWSKbdEvent(struct wscons_event *event)
 
     /* map the scancodes to standard XFree86 scancode */  	
     keycode = WSKbdToKeycode(value);
-    if (!down) keycode |= 0x80;
-    /* It seems better to block SIGIO there */
-    blocked = xf86BlockSIGIO();
-    xf86PostKbdEvent(keycode);
-    xf86UnblockSIGIO(blocked);
+    if ((keycode != KEY_NOTUSED) && ((keycode & 0x80) == 0)) {
+        if (!down) keycode |= 0x80;
+        /* It seems better to block SIGIO there */
+        blocked = xf86BlockSIGIO();
+        xf86PostKbdEvent(keycode);
+        xf86UnblockSIGIO(blocked);
+    }
   }
 }
 #endif /* WSCONS_SUPPORT */
