@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiload.c,v 1.18 2004/12/31 16:07:06 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiload.c,v 1.23 2006/03/16 16:49:58 dawes Exp $ */
 /*
- * Copyright 2000 through 2005 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 2000 through 2006 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -28,6 +28,8 @@
 #include "atiload.h"
 #include "atistruct.h"
 
+#include "vbe.h"
+
 /*
  * All symbol lists belong here.  They are externalised so that they can be
  * referenced elsewhere.  Note the naming convention for these things...
@@ -56,8 +58,6 @@ const char *ATIvbeSymbols[] =
     NULL
 };
 
-#ifndef AVOID_CPIO
-
 const char *ATIxf1bppSymbols[] =
 {
     "xf1bppScreenInit",
@@ -70,8 +70,6 @@ const char *ATIxf4bppSymbols[] =
     NULL
 };
 
-#endif /* AVOID_CPIO */
-
 const char *ATIfbSymbols[] =
 {
     "fbPictureInit",
@@ -114,34 +112,53 @@ const char *ATIi2cSymbols[] =
 };
 
 /*
- * ATILoadModule --
+ * ATILoadSubModule --
  *
  * Load a specific module and register with the loader those of its entry
  * points that are referenced by this driver.
  */
-pointer
-ATILoadModule
+ModuleDescPtr
+ATILoadSubModule
 (
     ScrnInfoPtr  pScreenInfo,
     const char  *Module,
     const char **SymbolList
 )
 {
-    pointer pModule = xf86LoadSubModule(pScreenInfo, Module);
+    ModuleDescPtr pModule = xf86LoadSubModule(pScreenInfo, Module);
 
     if (pModule)
-        xf86LoaderReqSymLists(SymbolList, NULL);
+        xf86LoaderModReqSymLists(pModule, SymbolList, NULL);
 
     return pModule;
 }
 
 /*
- * ATILoadModules --
+ * ATILoadVBEModule --
+ *
+ * A variant of ATILoadSubModule() specifically for loading the 'vbe' module.
+ */
+ModuleDescPtr
+ATILoadVBEModule
+(
+    ScrnInfoPtr  pScreenInfo
+)
+{
+    ModuleDescPtr pModule = xf86LoadVBEModule(pScreenInfo);
+
+    if (pModule)
+        xf86LoaderModReqSymLists(pModule, ATIvbeSymbols, NULL);
+
+    return pModule;
+}
+
+/*
+ * ATILoadSubModules --
  *
  * This function loads other modules required for a screen.
  */
-pointer
-ATILoadModules
+ModuleDescPtr
+ATILoadSubModules
 (
     ScrnInfoPtr pScreenInfo,
     ATIPtr      pATI
@@ -149,38 +166,33 @@ ATILoadModules
 {
     /* Load shadow frame buffer code if needed */
     if (pATI->OptionShadowFB &&
-        !ATILoadModule(pScreenInfo, "shadowfb", ATIshadowfbSymbols))
+        !ATILoadSubModule(pScreenInfo, "shadowfb", ATIshadowfbSymbols))
         return NULL;
 
     /* Load XAA if needed */
     if (pATI->OptionAccel &&
-        !ATILoadModule(pScreenInfo, "xaa", ATIxaaSymbols))
+        !ATILoadSubModule(pScreenInfo, "xaa", ATIxaaSymbols))
         return NULL;
 
     /* Load ramdac module if needed */
     if ((pATI->Cursor > ATI_CURSOR_SOFTWARE) &&
-        !ATILoadModule(pScreenInfo, "ramdac", ATIramdacSymbols))
+        !ATILoadSubModule(pScreenInfo, "ramdac", ATIramdacSymbols))
         return NULL;
 
     /* Load depth-specific entry points */
     switch (pATI->bitsPerPixel)
     {
-
-#ifndef AVOID_CPIO
-
         case 1:
-            return ATILoadModule(pScreenInfo, "xf1bpp", ATIxf1bppSymbols);
+            return ATILoadSubModule(pScreenInfo, "xf1bpp", ATIxf1bppSymbols);
 
         case 4:
-            return ATILoadModule(pScreenInfo, "xf4bpp", ATIxf4bppSymbols);
-
-#endif /* AVOID_CPIO */
+            return ATILoadSubModule(pScreenInfo, "xf4bpp", ATIxf4bppSymbols);
 
         case 8:
         case 16:
         case 24:
         case 32:
-            return ATILoadModule(pScreenInfo, "fb", ATIfbSymbols);
+            return ATILoadSubModule(pScreenInfo, "fb", ATIfbSymbols);
 
         default:
             return NULL;
