@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atimach64cursor.c,v 1.3 2004/12/31 16:07:06 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atimach64cursor.c,v 1.6 2006/01/05 18:55:29 tsi Exp $ */
 /*
- * Copyright 2003 through 2005 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 2003 through 2006 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -337,8 +337,6 @@ ATIMach64UseHWCursor
     if (!pATI->CursorBase)
         return FALSE;
 
-#ifndef AVOID_CPIO
-
     /*
      * For some reason, the hardware cursor isn't vertically scaled when a VGA
      * doublescanned or multiscanned mode is in effect.
@@ -349,8 +347,6 @@ ATIMach64UseHWCursor
         (pScreenInfo->currentMode->VScan > 1))
         return FALSE;
 
-#endif /* AVOID_CPIO */
-
     return TRUE;
 }
 
@@ -379,14 +375,9 @@ ATIMach64CursorInit
         HARDWARE_CURSOR_SHOW_TRANSPARENT |
         HARDWARE_CURSOR_UPDATE_UNHIDDEN |
         HARDWARE_CURSOR_AND_SOURCE_WITH_MASK |
-
-#if X_BYTE_ORDER != X_LITTLE_ENDIAN
-
-        HARDWARE_CURSOR_BIT_ORDER_MSBFIRST |
-
-#endif /* X_BYTE_ORDER */
-
         HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_1;
+    if (ATIEndian.endian == ATI_BIG_ENDIAN)
+        pCursorInfo->Flags |= HARDWARE_CURSOR_BIT_ORDER_MSBFIRST;
     pCursorInfo->MaxWidth = pCursorInfo->MaxHeight = 64;
 
     pCursorInfo->SetCursorColors = ATIMach64SetCursorColours;
