@@ -30,7 +30,7 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  * Copyright 2002 Shigehiro Nomura
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/neomagic/neo_driver.c,v 1.75 2004/02/18 04:20:30 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/neomagic/neo_driver.c,v 1.82 2006/03/16 16:50:10 dawes Exp $ */
 
 /*
  * The original Precision Insight driver for
@@ -95,7 +95,7 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "picturestr.h"
 
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 
 /*
  * Driver data structures.
@@ -105,12 +105,12 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "neo_macros.h"
 
 /* These need to be checked */
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "scrnintstr.h"
 #include "servermd.h"
 #define _XF86DGA_SERVER_
-#include "extensions/xf86dgastr.h"
+#include <X11/extensions/xf86dgastr.h>
 
 /* Mandatory functions */
 static const OptionInfoRec *	NEOAvailableOptions(int chipid, int busid);
@@ -142,7 +142,7 @@ static void     neoCalcVCLK(ScrnInfoPtr pScrn, long freq);
 static xf86MonPtr  neo_ddc1(int scrnIndex);
 static Bool     neoDoDDC1(ScrnInfoPtr pScrn);
 static Bool     neoDoDDC2(ScrnInfoPtr pScrn);
-static Bool     neoDoDDCVBE(ScrnInfoPtr pScrn);
+static Bool     neoDoDDCVBE(ScrnInfoPtr pScrn, ModuleDescPtr pDDCMod);
 static void     neoProbeDDC(ScrnInfoPtr pScrn, int index);
 static void     NeoDisplayPowerManagementSet(ScrnInfoPtr pScrn,
 				int PowerManagementMode, int flags);
@@ -413,8 +413,8 @@ static const OptionInfoRec NEOOptions[] = {
  * List of symbols from other modules that this module references.  This
  * list is used to tell the loader that it is OK for symbols here to be
  * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
+ * told that they are essential via a call to xf86LoaderModReqSymbols() or
+ * xf86LoaderModReqSymLists().  The purpose is this is to avoid warnings about
  * unresolved symbols that are not required.
  */
 
@@ -507,7 +507,7 @@ static XF86ModuleVersionInfo neoVersRec =
 XF86ModuleData neomagicModuleData = { &neoVersRec, neoSetup, NULL };
 
 static pointer
-neoSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+neoSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -524,9 +524,9 @@ neoSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 	 * Tell the loader about symbols from other modules that this module
 	 * might refer to.
 	 */
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols,
-			  ramdacSymbols, shadowSymbols,
-			  ddcSymbols, vbeSymbols, i2cSymbols, NULL);
+	LoaderModRefSymLists(module, vgahwSymbols, fbSymbols, xaaSymbols,
+			     ramdacSymbols, shadowSymbols,
+			     ddcSymbols, vbeSymbols, i2cSymbols, NULL);
 	/*
 	 * The return value must be non-NULL on success even though there
 	 * is no TearDownProc.
@@ -725,6 +725,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     Bool height_480 = FALSE;
     Bool lcdCenterOptSet = FALSE;
     char *s;
+    ModuleDescPtr pMod;
     
     if (flags & PROBE_DETECT)  {
 	neoProbeDDC( pScrn, xf86GetEntityInfo(pScrn->entityList[0])->index );
@@ -732,10 +733,10 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     }
     
     /* The vgahw module should be loaded here when needed */
-    if (!xf86LoadSubModule(pScrn, "vgahw"))
+    if (!(pMod = xf86LoadSubModule(pScrn, "vgahw")))
 	return FALSE;
 
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);    
+    xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);    
 
     /*
      * Allocate a vgaHWRec.
@@ -760,7 +761,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
      */
     /* This driver doesn't expect more than one entity per screen */
     if (pScrn->numEntities > 1)
-	RETURN;
+	RETURN
 
     /* This is the general case */
     for (i = 0; i<pScrn->numEntities; i++) {
@@ -953,10 +954,10 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 
     pScrn->monitor = pScrn->confScreen->monitor;
 
-    if (xf86LoadSubModule(pScrn, "ddc")) {
-        xf86LoaderReqSymLists(ddcSymbols, NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, "ddc"))) {
+        xf86LoaderModReqSymLists(pMod, ddcSymbols, NULL);
 #if 1 /* for DDC1 testing */
-	if (!neoDoDDCVBE(pScrn))
+	if (!neoDoDDCVBE(pScrn, pMod))
 	  if (!neoDoDDC2(pScrn))
 #endif
 	      neoDoDDC1(pScrn);
@@ -1250,7 +1251,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	}
 	/* XXX What about VGA resources in OPERATING mode? */
 	if (xf86RegisterResources(nPtr->pEnt->index, NULL, ResExclusive))
-	    RETURN;
+	    RETURN
 	    
     } else if (nPtr->pEnt->location.type == BUS_ISA) {
 	unsigned int addr;
@@ -1277,7 +1278,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	    nPtr->noLinear = TRUE; /* XXX */
 	}
     } else
-	RETURN;
+	RETURN
 
     if (nPtr->pEnt->device->videoRam != 0) {
 	pScrn->videoRam = nPtr->pEnt->device->videoRam;
@@ -1336,25 +1337,9 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	 * If the monitor parameters are not specified explicitly, set them
 	 * so that 60Hz modes up to the panel size are allowed.
 	 */
-	if (pScrn->monitor->nHsync == 0) {
-	    pScrn->monitor->nHsync = 1;
-	    pScrn->monitor->hsync[0].lo = 28;
-	    pScrn->monitor->hsync[0].hi =
-				60.0 * 1.07 * nPtr->NeoPanelHeight / 1000.0;
-	    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
-		       "Using hsync range matching panel size: %.2f-%.2f kHz\n",
-		       pScrn->monitor->hsync[0].lo,
-		       pScrn->monitor->hsync[0].hi);
-	}
-	if (pScrn->monitor->nVrefresh == 0) {
-	    pScrn->monitor->nVrefresh = 1;
-	    pScrn->monitor->vrefresh[0].lo = 55.0;
-	    pScrn->monitor->vrefresh[0].hi = 65.0;
-	    xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
-		       "Using vsync range for panel: %.2f-%.2f kHz\n",
-		       pScrn->monitor->vrefresh[0].lo,
-		       pScrn->monitor->vrefresh[0].hi);
-	}
+	xf86SetMonitorParameters(pScrn, pScrn->monitor,
+				 nPtr->NeoPanelWidth, nPtr->NeoPanelHeight,
+				 60);
     }
 
     /*
@@ -1369,7 +1354,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 			  LOOKUP_BEST_REFRESH);
 
        if (i == -1)
-           RETURN;
+           RETURN
     }
 
     /* Prune the modes marked as invalid */
@@ -1377,7 +1362,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 
     if (i == 0 || pScrn->modes == NULL) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No valid modes found\n");
-	RETURN;
+	RETURN
     }
 
     /*
@@ -1399,29 +1384,29 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     /* If monitor resolution is set on the command line, use it */
     xf86SetDpi(pScrn, 0, 0);
 
-    if (xf86LoadSubModule(pScrn, "fb") == NULL) {
-	RETURN;
+    if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
+	RETURN
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
 
     if (!nPtr->noLinear) {
-	if (!xf86LoadSubModule(pScrn, "xaa")) 
-	    RETURN;
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
+	if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) 
+	    RETURN
+	xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
     }
 
     if (nPtr->shadowFB) {
-	if (!xf86LoadSubModule(pScrn, "shadow")) {
-	    RETURN;
+	if (!(pMod = xf86LoadSubModule(pScrn, "shadow"))) {
+	    RETURN
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, shadowSymbols, NULL);
     }
     
     if (!nPtr->swCursor) {
-	if (!xf86LoadSubModule(pScrn, "ramdac"))
-	    RETURN;
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
+	if (!(pMod = xf86LoadSubModule(pScrn, "ramdac")))
+	    RETURN
+	xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
     }
     return TRUE;
 }
@@ -1970,7 +1955,7 @@ NEOValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 			   mode->VDisplay,
 			   nPtr->NeoPanelWidth,
 			   nPtr->NeoPanelHeight);
-		return(MODE_BAD);
+		return(MODE_PANEL);
 	    }
 
 	    /* Is the mode one of the acceptable sizes? */
@@ -2009,7 +1994,7 @@ NEOValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 		       "display properly on LCD\n",
 		       mode->HDisplay,
 		       mode->VDisplay);
-	    return(MODE_BAD);
+	    return(MODE_PANEL_NOSCALE);
 	}
     }
     return(MODE_OK);
@@ -2354,7 +2339,6 @@ neoProgramShadowRegs(ScrnInfoPtr pScrn, vgaRegPtr VgaReg, NeoRegPtr restore)
 		VGAwCR(0x58,0x2C);
 		VGAwCR(0x59,0x94);
 		break;
-		break;
 		/* Not done */
 	    }
 	    break;
@@ -2659,6 +2643,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
      * generic VGA registers.
      */
 
+    hwp->Flags |= VGA_FIX_SYNC_PULSES;
     if (!vgaHWInit(pScrn, mode))
 	return(FALSE);
 
@@ -2702,10 +2687,11 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     NeoNew->ExtCRTDispAddr = 0x10;
 
     /* Vertical Extension */
-    NeoNew->VerticalExt = (((mode->CrtcVTotal -2) & 0x400) >> 10 )
-      | (((mode->CrtcVDisplay -1) & 0x400) >> 9 )
-        | (((mode->CrtcVSyncStart) & 0x400) >> 8 )
-          | (((mode->CrtcVSyncStart) & 0x400) >> 7 );
+    /* XXX Are Sync & Blank flipped here? */
+    NeoNew->VerticalExt = (((mode->CrtcVTotal - 2) & 0x400) >> 10)
+      | (((mode->CrtcVDisplay - 1) & 0x400) >> 9)
+        | (((mode->CrtcVSyncStart - 1) & 0x400) >> 8)
+          | (((mode->CrtcVBlankStart - 1) & 0x400) >> 7);
 
     /* Fast write bursts on unless disabled. */
     if (nPtr->onPciBurst) {
@@ -3135,10 +3121,11 @@ neoDoDDC2(ScrnInfoPtr pScrn)
     NEOPtr nPtr = NEOPTR(pScrn);    
     vgaHWPtr hwp = VGAHWPTR(pScrn);
     Bool ret = FALSE;
+    ModuleDescPtr pMod;
 
     VGAwGR(0x09,0x26);
-    if (xf86LoadSubModule(pScrn, "i2c")) {
-        xf86LoaderReqSymLists(i2cSymbols, NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, "i2c"))) {
+        xf86LoaderModReqSymLists(pMod, i2cSymbols, NULL);
 	if (neo_I2CInit(pScrn)) {
 	    ret = xf86SetDDCproperties(pScrn,xf86PrintEDID(xf86DoEDID_DDC2(
 					      pScrn->scrnIndex,nPtr->I2C)));
@@ -3150,21 +3137,23 @@ neoDoDDC2(ScrnInfoPtr pScrn)
 }
 
 static Bool
-neoDoDDCVBE(ScrnInfoPtr pScrn)
+neoDoDDCVBE(ScrnInfoPtr pScrn, ModuleDescPtr pDDCMod)
 {
     NEOPtr nPtr = NEOPTR(pScrn);    
     vgaHWPtr hwp = VGAHWPTR(pScrn);
     vbeInfoPtr pVbe;
     Bool ret = FALSE;
+    ModuleDescPtr pMod;
 
     VGAwGR(0x09,0x26);
-    if (xf86LoadSubModule(pScrn, "vbe")) {
-	xf86LoaderReqSymLists(vbeSymbols, NULL);
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
         if ((pVbe = VBEInit(NULL,nPtr->pEnt->index))) {
-	  ret = xf86SetDDCproperties(
-				     pScrn,xf86PrintEDID(vbeDoEDID(pVbe,NULL)));
-	  vbeFree(pVbe);
+	    ret = xf86SetDDCproperties(pScrn,
+				       xf86PrintEDID(vbeDoEDID(pVbe, pDDCMod)));
+	    vbeFree(pVbe);
 	}
+	xf86UnloadSubModule(pMod);
     }
     VGAwGR(0x09,0x00);
     return ret;
@@ -3217,11 +3206,14 @@ static void
 neoProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
+    ModuleDescPtr pMod;
 
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
         if ((pVbe = VBEInit(NULL,index))) {
 	    ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	    vbeFree(pVbe);
 	}
+	xf86UnloadSubModule(pMod);
     }
 }
