@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suncg6/cg6_driver.c,v 1.12 2005/02/18 02:55:09 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suncg6/cg6_driver.c,v 1.15 2006/03/16 16:50:16 dawes Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -89,9 +89,33 @@ typedef enum {
 static const OptionInfoRec CG6Options[] = {
     { OPTION_SW_CURSOR,		"SWcursor",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_HW_CURSOR,		"HWcursor",	OPTV_BOOLEAN,	{0}, FALSE },
+    { OPTION_NOACCEL,		"NoAccel",	OPTV_BOOLEAN,	{0}, FALSE },
     { -1,			NULL,		OPTV_NONE,	{0}, FALSE }
 };
 
+static const char *fbSymbols[] =
+{
+    "fbPictureInit",
+    "fbScreenInit",
+    NULL
+};
+
+static const char *ramdacSymbols[] =
+{
+    "xf86CreateCursorInfoRec",
+    "xf86DestroyCursorInfoRec",
+    "xf86InitCursor",
+    NULL
+};
+
+static const char *xaaSymbols[] =
+{
+    "XAACreateInfoRec",
+    "XAADestroyInfoRec",
+    "XAAInit",
+    NULL
+};
+
 #ifdef XFree86LOADER
 
 static MODULESETUPPROTO(cg6Setup);
@@ -113,7 +137,7 @@ static XF86ModuleVersionInfo suncg6VersRec =
 XF86ModuleData suncg6ModuleData = { &suncg6VersRec, cg6Setup, NULL };
 
 static pointer
-cg6Setup(pointer module, pointer opts, int *errmaj, int *errmin)
+cg6Setup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -125,6 +149,8 @@ cg6Setup(pointer module, pointer opts, int *errmaj, int *errmin)
 	 * Modules that this driver always requires can be loaded here
 	 * by calling LoadSubModule().
 	 */
+	xf86LoaderModRefSymLists(module, fbSymbols, ramdacSymbols,
+				 xaaSymbols, NULL);
 
 	/*
 	 * The return value must be non-NULL on success even though there
@@ -282,6 +308,7 @@ CG6PreInit(ScrnInfoPtr pScrn, int flags)
     sbusDevicePtr psdp;
     MessageType from;
     int i;
+    ModuleDescPtr pMod;
 
     if (flags & PROBE_DETECT) return FALSE;
 
@@ -381,14 +408,31 @@ CG6PreInit(ScrnInfoPtr pScrn, int flags)
     xf86DrvMsg(pScrn->scrnIndex, from, "Using %s cursor\n",
 		pCg6->HWCursor ? "HW" : "SW");
 
-    if (xf86LoadSubModule(pScrn, "fb") == NULL) {
+    if (xf86ReturnOptValBool(pCg6->Options, OPTION_NOACCEL, FALSE)) {
+	pCg6->NoAccel = TRUE;
+	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Acceleration disabled\n");
+    }
+
+    if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
 	CG6FreeRec(pScrn);
 	return FALSE;
     }
+    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
 
-    if (pCg6->HWCursor && xf86LoadSubModule(pScrn, "ramdac") == NULL) {
-	CG6FreeRec(pScrn);
-	return FALSE;
+    if (pCg6->HWCursor) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
+	    CG6FreeRec(pScrn);
+	    return FALSE;
+	}
+	xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
+    }
+
+    if (!pCg6->NoAccel) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
+	    CG6FreeRec(pScrn);
+	    return FALSE;
+	}
+	xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
     }
 
     /*********************
@@ -424,16 +468,37 @@ CG6ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     ScrnInfoPtr pScrn;
     Cg6Ptr pCg6;
     sbusDevicePtr psdp;
-    int ret;
+    int ret, vidmem;
 
     pScrn = xf86Screens[pScreen->myNum];
     pCg6 = GET_CG6_FROM_SCRN(pScrn);
     psdp = pCg6->psdp;
+    vidmem = psdp->width * psdp->height;
 
     /* Map CG6 memory areas */
     pCg6->fbc = xf86MapSbusMem(psdp, CG6_FBC_VOFF, sizeof(*pCg6->fbc));
     pCg6->thc = xf86MapSbusMem(psdp, CG6_THC_VOFF, sizeof(*pCg6->thc));
-    pCg6->fb = xf86MapSbusMem(psdp, CG6_RAM_VOFF, psdp->width * psdp->height);
+
+    do {
+	if (!pCg6->NoAccel) {
+	    /*
+	     * To allow for off-screen pixmaps, first try mapping 2MB, then
+	     * 1MB, followed by the minimum required.
+	     */
+	    pCg6->vidmem = max(2 * 1024 * 1024, vidmem);
+	    pCg6->fb = xf86MapSbusMem(psdp, CG6_RAM_VOFF, pCg6->vidmem);
+	    if (pCg6->fb || (pCg6->vidmem == vidmem))
+		break;
+
+	    pCg6->vidmem = max(1024 * 1024, vidmem);
+	    pCg6->fb = xf86MapSbusMem(psdp, CG6_RAM_VOFF, pCg6->vidmem);
+	    if (pCg6->fb || (pCg6->vidmem == vidmem))
+		break;
+	}
+
+	pCg6->vidmem = vidmem;
+	pCg6->fb = xf86MapSbusMem(psdp, CG6_RAM_VOFF, pCg6->vidmem);
+    } while (0);
 
     if (!pCg6->fbc || !pCg6->thc || !pCg6->fb) {
 	if (pCg6->fbc) {
@@ -447,13 +512,16 @@ CG6ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	}
 
 	if (pCg6->fb) {
-	    xf86UnmapSbusMem(psdp, pCg6->fb, psdp->width * psdp->height);
+	    xf86UnmapSbusMem(psdp, pCg6->fb, pCg6->vidmem);
 	    pCg6->fb = NULL;
 	}
 
 	return FALSE;
     }
 
+    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Mapped %d KB of video memory\n",
+	pCg6->vidmem >> 10);
+
     /* Darken the screen for aesthetic reasons and set the viewport */
     CG6SaveScreen(pScreen, SCREEN_SAVER_ON);
 
@@ -493,6 +561,10 @@ CG6ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!ret)
 	return FALSE;
 
+    pCg6->width = pScrn->virtualX;
+    pCg6->height = pScrn->virtualY;
+    pCg6->maxheight = (pCg6->vidmem / pCg6->width) & 0xffff;
+
     fbPictureInit(pScreen, 0, 0);
 
     miInitializeBackingStore(pScreen);
@@ -501,6 +573,21 @@ CG6ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     xf86SetBlackWhitePixels(pScreen);
 
+    if (!pCg6->NoAccel) {
+	BoxRec bx;
+
+	pCg6->pXAA = XAACreateInfoRec();
+	CG6AccelInit(pScrn);
+	bx.x1 = bx.y1 = 0;
+	bx.x2 = pCg6->width;
+	bx.y2 = pCg6->maxheight;
+	xf86InitFBManager(pScreen, &bx);
+	if(!XAAInit(pScreen, pCg6->pXAA))
+	    return FALSE;
+
+	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Using acceleration\n");
+    }
+
     /* Initialise cursor functions */
     miDCInitialize(pScreen, xf86GetPointerScreenFuncs());
 
@@ -604,19 +691,31 @@ CG6CloseScreen(int scrnIndex, ScreenPtr pScreen)
     ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
     Cg6Ptr pCg6 = GET_CG6_FROM_SCRN(pScrn);
     sbusDevicePtr psdp = pCg6->psdp;
+    Bool closed;
 
     pScrn->vtSema = FALSE;
 
-    xf86UnmapSbusMem(psdp, pCg6->fbc, sizeof(*pCg6->fbc));
-    xf86UnmapSbusMem(psdp, pCg6->thc, sizeof(*pCg6->thc));
-    xf86UnmapSbusMem(psdp, pCg6->fb, psdp->width * psdp->height);
-
     if (pCg6->HWCursor)
 	xf86SbusHideOsHwCursor(psdp);
 
+    if (pCg6->pXAA) {
+	XAADestroyInfoRec(pCg6->pXAA);
+	pCg6->pXAA = NULL;
+    }
+
     pScreen->CloseScreen = pCg6->CloseScreen;
-    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
-    return FALSE;
+    closed = (*pScreen->CloseScreen)(scrnIndex, pScreen);
+
+    if (pCg6->CursorInfoRec) {
+	xf86DestroyCursorInfoRec(pCg6->CursorInfoRec);
+	pCg6->CursorInfoRec = NULL;
+    }
+
+    xf86UnmapSbusMem(psdp, pCg6->fbc, sizeof(*pCg6->fbc));
+    xf86UnmapSbusMem(psdp, pCg6->thc, sizeof(*pCg6->thc));
+    xf86UnmapSbusMem(psdp, pCg6->fb, pCg6->vidmem);
+
+    return closed;
 }
 
 
