@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suntcx/tcx_driver.c,v 1.11 2005/02/18 02:55:10 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suntcx/tcx_driver.c,v 1.12 2006/03/16 16:50:17 dawes Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -113,7 +113,7 @@ static XF86ModuleVersionInfo suntcxVersRec =
 XF86ModuleData suntcxModuleData = { &suntcxVersRec, tcxSetup, NULL };
 
 static pointer
-tcxSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+tcxSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
