@@ -27,7 +27,7 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 **************************************************************************/
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/tdfx/tdfx_driver.c,v 1.105 2003/11/03 05:11:41 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/tdfx/tdfx_driver.c,v 1.114 2006/03/16 16:50:17 dawes Exp $ */
 
 /*
  * Authors:
@@ -81,7 +81,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 /* !!! These need to be checked !!! */
 #if 0
 #define _XF86DGA_SERVER_
-#include "extensions/xf86dgastr.h"
+#include <X11/extensions/xf86dgastr.h>
 #endif
 
 /* The driver's own header file: */
@@ -92,7 +92,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "dixstruct.h"
 
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 
 #ifdef XF86DRI
 #include "dri.h"
@@ -138,6 +138,8 @@ static void TDFXBlockHandler(int, pointer, pointer, pointer);
 static void TDFXDisplayPowerManagementSet(ScrnInfoPtr pScrn, 
 					int PowerManagermentMode, int flags);
 
+static xf86MonPtr doTDFXDDC(ScrnInfoPtr pScrn);
+
 DriverRec TDFX = {
   TDFX_VERSION,
   TDFX_DRIVER_NAME,
@@ -214,6 +216,14 @@ static const char *ramdacSymbols[] = {
 static const char *ddcSymbols[] = {
     "xf86PrintEDID",
     "xf86SetDDCproperties",
+    "xf86DoEDID_DDC2",
+    NULL
+};
+
+
+static const char *i2cSymbols[] = {
+    "xf86CreateI2CBusRec",
+    "xf86I2CBusInit",
     NULL
 };
 
@@ -264,6 +274,7 @@ static const char *driSymbols[] = {
     "DRIScreenInit",
     "DRIUnlock",
     "GlxSetVisualConfigs",
+    "DRICreatePCIBusID",
     NULL
 };
 
@@ -290,7 +301,7 @@ static XF86ModuleVersionInfo tdfxVersRec =
 XF86ModuleData tdfxModuleData = {&tdfxVersRec, tdfxSetup, 0};
 
 static pointer
-tdfxSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+tdfxSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -309,12 +320,12 @@ tdfxSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 	 * Tell the loader about symbols from other modules that this module
 	 * might refer to.
 	 */
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols, 
-			  ramdacSymbols, vbeSymbols, int10Symbols,
+	LoaderModRefSymLists(module, vgahwSymbols, fbSymbols, xaaSymbols, 
+			     ramdacSymbols, vbeSymbols, int10Symbols,
 #ifdef XF86DRI
-			  drmSymbols, driSymbols,
+			     drmSymbols, driSymbols,
 #endif
-			  NULL);
+			     NULL);
 
 	/*
 	 * The return value must be non-NULL on success even though there
@@ -374,11 +385,14 @@ static void
 TDFXProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
-    if (xf86LoadSubModule(pScrn, "vbe"))
+    ModuleDescPtr pMod;
+    if ((pMod = xf86LoadVBEModule(pScrn)))
     {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
 	pVbe =  VBEInit(NULL,index);
 	ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
     }
 }
 
@@ -673,12 +687,14 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
 {
   TDFXPtr pTDFX;
   ClockRangePtr clockRanges;
+  xf86MonPtr pMon;
   int i;
   MessageType from;
   int flags24;
   rgb defaultWeight = {0, 0, 0};
   pciVideoPtr match;
   int availableMem;
+  ModuleDescPtr pMod, pDDCMod;
 
   TDFXTRACE("TDFXPreInit start\n");
   if (pScrn->numEntities != 1) return FALSE;
@@ -705,19 +721,19 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
   if (pTDFX->pEnt->location.type != BUS_PCI) return FALSE;
 
   /* The vgahw module should be loaded here when needed */
-  if (!xf86LoadSubModule(pScrn, "vgahw")) return FALSE;
+  if (!(pMod = xf86LoadSubModule(pScrn, "vgahw"))) return FALSE;
 
-  xf86LoaderReqSymLists(vgahwSymbols, NULL);
+  xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);
 
   /* Allocate a vgaHWRec */
   if (!vgaHWGetHWRec(pScrn)) return FALSE;
 
 #if USE_INT10
 #if !defined(__powerpc__)
-  if (xf86LoadSubModule(pScrn, "int10")) {
+  if ((pMod = xf86LoadSubModule(pScrn, "int10"))) {
     xf86Int10InfoPtr pInt;
 
-    xf86LoaderReqSymLists(int10Symbols, NULL);
+    xf86LoaderModReqSymLists(pMod, int10Symbols, NULL);
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
                "Softbooting the board (through the int10 interface).\n");
     pInt = xf86InitInt10(pTDFX->pEnt->index);
@@ -974,6 +990,88 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
   availableMem = pScrn->videoRam - 4096 -
 		 (((255 <= CMDFIFO_PAGES) ? 255 : CMDFIFO_PAGES) << 12);
 
+  xf86SetDpi(pScrn, 0, 0);
+
+  if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
+    TDFXFreeRec(pScrn);
+    return FALSE;
+  }
+  xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
+
+  if (!xf86ReturnOptValBool(pTDFX->Options, OPTION_NOACCEL, FALSE)) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
+      TDFXFreeRec(pScrn);
+      return FALSE;
+    }
+    xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
+  }
+
+  if (!xf86GetOptValBool(pTDFX->Options, OPTION_SHOWCACHE, &(pTDFX->ShowCache))) {
+    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ShowCache %s\n", pTDFX->ShowCache ? "Enabled" : "Disabled");
+  } else {
+    pTDFX->ShowCache = FALSE;
+  }
+
+  if (xf86GetOptValBool(pTDFX->Options, OPTION_TEXTURED_VIDEO, &(pTDFX->TextureXvideo))) {
+    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Texture Xvideo Adaptor %s\n", pTDFX->TextureXvideo ? "Enabled" : "Disabled");
+  } else {
+    pTDFX->TextureXvideo = FALSE;
+  }
+
+  if (xf86GetOptValInteger(pTDFX->Options, OPTION_VIDEO_KEY, &(pTDFX->videoKey))) {
+    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "video key set to 0x%x\n", pTDFX->videoKey);
+  } else {
+    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "video key default 0x%x\n", pTDFX->videoKey = 0x1E);
+  }
+
+  if (!xf86ReturnOptValBool(pTDFX->Options, OPTION_SW_CURSOR, FALSE)) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
+      TDFXFreeRec(pScrn);
+      return FALSE;
+    }
+    xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
+  }
+
+  /* Load DDC and I2C for monitor ID */
+  if (!(pMod = xf86LoadSubModule(pScrn, "i2c"))) {
+    TDFXFreeRec(pScrn);
+    return FALSE;
+  }
+  xf86LoaderModReqSymLists(pMod, i2cSymbols, NULL);
+
+  if (!(pDDCMod = xf86LoadSubModule(pScrn, "ddc"))) {
+    TDFXFreeRec(pScrn);
+    return FALSE;
+  }
+  xf86LoaderModReqSymLists(pDDCMod, ddcSymbols, NULL);
+
+  /* try to read read DDC2 data */
+  pMon = doTDFXDDC(pScrn);
+  if (pMon != NULL) {
+    xf86SetDDCproperties(pScrn,xf86PrintEDID(pMon));
+#if USE_INT10
+#if !defined(__powerpc__)
+  } else {
+    /* try to use vbe if we didn't find anything */
+    /* Initialize DDC1 if possible */
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+        vbeInfoPtr pVbe = VBEInit(NULL,pTDFX->pEnt->index);
+        xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
+        if (pVbe) {
+            pMon = vbeDoEDID(pVbe, pDDCMod);
+            vbeFree(pVbe);
+            xf86SetDDCproperties(pScrn,xf86PrintEDID(pMon));
+        }
+        xf86UnloadSubModule(pMod);
+    }
+#endif
+#endif
+  }
+
+  if (xf86ReturnOptValBool(pTDFX->Options, OPTION_USE_PIO, FALSE)) {
+    pTDFX->usePIO=TRUE;
+  }
+
   i = xf86ValidateModes(pScrn, pScrn->monitor->Modes,
 			pScrn->display->modes, clockRanges,
 			0, 320, 2048, 16*pScrn->bitsPerPixel, 
@@ -1000,75 +1098,6 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
 
   xf86PrintModes(pScrn);
 
-  xf86SetDpi(pScrn, 0, 0);
-
-  if (!xf86LoadSubModule(pScrn, "fb")) {
-    TDFXFreeRec(pScrn);
-    return FALSE;
-  }
-  xf86LoaderReqSymLists(fbSymbols, NULL);
-
-  if (!xf86ReturnOptValBool(pTDFX->Options, OPTION_NOACCEL, FALSE)) {
-    if (!xf86LoadSubModule(pScrn, "xaa")) {
-      TDFXFreeRec(pScrn);
-      return FALSE;
-    }
-    xf86LoaderReqSymLists(xaaSymbols, NULL);
-  }
-
-  if (!xf86GetOptValBool(pTDFX->Options, OPTION_SHOWCACHE, &(pTDFX->ShowCache))) {
-    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ShowCache %s\n", pTDFX->ShowCache ? "Enabled" : "Disabled");
-  } else {
-    pTDFX->ShowCache = FALSE;
-  }
-
-  if (xf86GetOptValBool(pTDFX->Options, OPTION_TEXTURED_VIDEO, &(pTDFX->TextureXvideo))) {
-    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Texture Xvideo Adaptor %s\n", pTDFX->TextureXvideo ? "Enabled" : "Disabled");
-  } else {
-    pTDFX->TextureXvideo = FALSE;
-  }
-
-  if (xf86GetOptValInteger(pTDFX->Options, OPTION_VIDEO_KEY, &(pTDFX->videoKey))) {
-    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "video key set to 0x%x\n", pTDFX->videoKey);
-  } else {
-    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "video key default 0x%x\n", pTDFX->videoKey = 0x1E);
-  }
-
-  if (!xf86ReturnOptValBool(pTDFX->Options, OPTION_SW_CURSOR, FALSE)) {
-    if (!xf86LoadSubModule(pScrn, "ramdac")) {
-      TDFXFreeRec(pScrn);
-      return FALSE;
-    }
-    xf86LoaderReqSymLists(ramdacSymbols, NULL);
-  }
-
-#if USE_INT10
-#if !defined(__powerpc__)
-  /* Load DDC if needed */
-  /* This gives us DDC1 - we should be able to get DDC2B using i2c */
-  if (!xf86LoadSubModule(pScrn, "ddc")) {
-    TDFXFreeRec(pScrn);
-    return FALSE;
-  }
-  xf86LoaderReqSymLists(ddcSymbols, NULL);
-
-  /* Initialize DDC1 if possible */
-  if (xf86LoadSubModule(pScrn, "vbe")) {
-      xf86MonPtr pMon;
-      vbeInfoPtr pVbe = VBEInit(NULL,pTDFX->pEnt->index);
-
-      xf86LoaderReqSymLists(vbeSymbols, NULL);
-      pMon = vbeDoEDID(pVbe, NULL);
-      vbeFree(pVbe);
-      xf86SetDDCproperties(pScrn,xf86PrintEDID(pMon));
-  }
-#endif
-#endif
-
-  if (xf86ReturnOptValBool(pTDFX->Options, OPTION_USE_PIO, FALSE)) {
-    pTDFX->usePIO=TRUE;
-  }
-
 #if X_BYTE_ORDER == X_BIG_ENDIAN
   pTDFX->ModeReg.miscinit0 = pTDFX->readLong(pTDFX, MISCINIT0);
   pTDFX->SavedReg.miscinit0 = pTDFX->ModeReg.miscinit0;
@@ -1101,8 +1130,8 @@ TDFXPreInit(ScrnInfoPtr pScrn, int flags)
 #ifdef XF86DRI
   /* Load the dri module if requested. */
   if (xf86ReturnOptValBool(pTDFX->Options, OPTION_DRI, FALSE)) {
-    if (xf86LoadSubModule(pScrn, "dri")) {
-      xf86LoaderReqSymLists(driSymbols, drmSymbols, NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, "dri"))) {
+      xf86LoaderModReqSymLists(pMod, driSymbols, drmSymbols, NULL);
     }
   }
 #endif
@@ -1636,8 +1665,8 @@ TDFXSetMode(ScrnInfoPtr pScrn, DisplayModePtr mode) {
 
   /* Calculate the CRTC values */
   hd = (mode->CrtcHDisplay>>3)-1;
-  hss = (mode->CrtcHSyncStart>>3);
-  hse = (mode->CrtcHSyncEnd>>3);
+  hss = (mode->CrtcHSyncStart>>3)-1;
+  hse = (mode->CrtcHSyncEnd>>3)-1;
   ht = (mode->CrtcHTotal>>3)-5;
   hbs = (mode->CrtcHBlankStart>>3)-1;
   hbe = (mode->CrtcHBlankEnd>>3)-1;
@@ -1717,6 +1746,7 @@ TDFXModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
   vgaHWUnlock(hwp);
 
+  hwp->Flags |= VGA_FIX_SYNC_PULSES;
   if (!vgaHWInit(pScrn, mode)) return FALSE;
 
   pScrn->vtSema = TRUE;
@@ -2405,28 +2435,27 @@ TDFXValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags) {
   TDFXPtr pTDFX;
 
   TDFXTRACE("TDFXValidMode start\n");
-  if ((mode->HDisplay>2048) || (mode->VDisplay>1536)) 
-    return MODE_BAD;
+  if (mode->HDisplay>2048)
+    return MODE_BAD_HVALUE;
+  else if (mode->VDisplay>1536) 
+    return MODE_BAD_VVALUE;
   /* Banshee doesn't support interlace, but Voodoo 3 and higher do. */
   pScrn = xf86Screens[scrnIndex];
   pTDFX = TDFXPTR(pScrn);
   if (mode->Flags&V_INTERLACE) {
     switch (pTDFX->ChipType) {
       case PCI_CHIP_BANSHEE:
-        return MODE_BAD;
-        break;
+        return MODE_NO_INTERLACE;
       case PCI_CHIP_VOODOO3:
       case PCI_CHIP_VOODOO5:
         return MODE_OK;
-        break;
       default:
-        return MODE_BAD;
-        break;
+        return MODE_NO_INTERLACE;
     }
   }
   /* In clock doubled mode widths must be divisible by 16 instead of 8 */
   if ((mode->Clock>TDFX2XCUTOFF) && (mode->HDisplay%16))
-    return MODE_BAD;
+    return MODE_H_ILLEGAL;
   return MODE_OK;
 }
 
@@ -2520,3 +2549,69 @@ TDFXDisplayPowerManagementSet(ScrnInfoPtr pScrn, int PowerManagementMode,
   dacmode|=state;
   pTDFX->writeLong(pTDFX, DACMODE, dacmode);
 }
+
+static void
+TDFXPutBits(I2CBusPtr b, int  scl, int  sda)
+{
+  TDFXPtr pTDFX;
+  CARD32 reg;
+  
+  pTDFX = b->DriverPrivate.ptr;
+  reg = pTDFX->readLong(pTDFX, VIDSERIALPARALLELPORT);
+  reg = (reg & ~(VSP_SDA0_OUT | VSP_SCL0_OUT)) |
+  	(scl ? VSP_SCL0_OUT : 0) |
+  	(sda ? VSP_SDA0_OUT : 0);
+  pTDFX->writeLong(pTDFX, VIDSERIALPARALLELPORT, reg);
+}
+
+static void
+TDFXGetBits(I2CBusPtr b, int *scl, int *sda)
+{
+  TDFXPtr pTDFX;
+  CARD32 reg;
+  
+  pTDFX = b->DriverPrivate.ptr;
+  reg = pTDFX->readLong(pTDFX, VIDSERIALPARALLELPORT);
+  *sda = (reg & VSP_SDA0_IN) ? 1 : 0;
+  *scl = (reg & VSP_SCL0_IN) ? 1 : 0;
+}
+
+static xf86MonPtr
+doTDFXDDC(ScrnInfoPtr pScrn)
+{
+  TDFXPtr pTDFX;
+  I2CBusPtr pI2CBus;
+  xf86MonPtr pMon = NULL;
+  CARD32 reg;
+  
+  pTDFX = TDFXPTR(pScrn);
+
+  if (!(pI2CBus = xf86CreateI2CBusRec()))
+  {
+    xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "Unable to allocate I2C Bus record.\n");
+    return NULL;
+  }
+
+  /* Fill in generic structure fields */
+  pI2CBus->BusName           = "DDC";
+  pI2CBus->scrnIndex         = pScrn->scrnIndex;
+  pI2CBus->I2CPutBits        = TDFXPutBits;
+  pI2CBus->I2CGetBits        = TDFXGetBits;
+  
+  pI2CBus->DriverPrivate.ptr = pTDFX;
+
+  reg = pTDFX->readLong(pTDFX, VIDSERIALPARALLELPORT);
+  pTDFX->writeLong(pTDFX, VIDSERIALPARALLELPORT, reg | VSP_ENABLE_IIC0);
+
+  if (xf86I2CBusInit(pI2CBus))
+  {
+    pMon = xf86DoEDID_DDC2(pScrn->scrnIndex, pI2CBus);
+    if (pMon == NULL)
+      xf86Msg(X_WARNING, "No DDC2 capable monitor found\n");
+    xf86DestroyI2CBusRec(pI2CBus, TRUE, TRUE);
+  }
+  pTDFX->writeLong(pTDFX, VIDSERIALPARALLELPORT, reg);
+  
+  return pMon;
+}
+
