@@ -2,7 +2,7 @@
  *  video4linux Xv Driver 
  *  based on Michael Schimek's permedia 2 driver.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/v4l/v4l.c,v 1.34 2003/12/31 06:08:53 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/v4l/v4l.c,v 1.36 2006/03/16 16:50:20 dawes Exp $ */
 
 #include "videodev.h"
 #include "xf86.h"
@@ -12,7 +12,7 @@
 #include "xf86PciInfo.h"
 #include "xf86fbman.h"
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 #include "regionstr.h"
 #include "dgaproc.h"
 #include "xf86str.h"
@@ -61,7 +61,7 @@ static XF86ModuleVersionInfo v4lVersRec =
 XF86ModuleData v4lModuleData = { &v4lVersRec, v4lSetup, NULL };
 
 static pointer
-v4lSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+v4lSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
         const char *osname;
 	static Bool setupDone = FALSE;
