@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/int10/xf86int10module.c,v 1.2 2002/09/16 18:06:09 eich Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/int10/xf86int10module.c,v 1.3 2006/03/16 16:50:33 dawes Exp $ */
 /*
  *                   XFree86 int10 module
  *   execute BIOS int 10h calls in x86 real mode environment
@@ -42,7 +42,7 @@ static XF86ModuleVersionInfo NAME(VersRec) =
 XF86ModuleData NAME(ModuleData) = { &NAME(VersRec), NAME(Setup), NULL };
 
 static pointer
-NAME(Setup)(pointer module, pointer opts, int *errmaj, int *errmin)
+NAME(Setup)(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
     
