@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/shared/libc_wrapper.c,v 1.103 2004/02/13 23:58:48 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/shared/libc_wrapper.c,v 1.111 2006/01/29 02:19:54 tsi Exp $ */
 /*
- * Copyright 1997-2003 by The XFree86 Project, Inc.
+ * Copyright 1997-2005 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -49,12 +49,12 @@
 #if defined(linux) && !defined(__GLIBC__)
 #undef __STRICT_ANSI__
 #endif
-#include <X.h>
+#include <X11/X.h>
 #ifdef __UNIXOS2__
 #define I_NEED_OS2_H
 #endif
-#include <Xmd.h>
-#include <Xos.h>
+#include <X11/Xmd.h>
+#include <X11/Xos.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #if defined(__bsdi__)
@@ -68,7 +68,7 @@
 #endif
 #include <stdarg.h>
 #include <fcntl.h>
-#include "Xfuncproto.h"
+#include <X11/Xfuncproto.h>
 #include "os.h"
 #include <ctype.h>
 #include <unistd.h>
@@ -77,11 +77,9 @@
 #include <stdio.h>
 #include <sys/ioctl.h>
 #ifdef __UNIXOS2__
-#define NO_MMAP
 #include <sys/param.h>
 #endif
 #ifdef HAS_SVR3_MMAPDRV
-#define NO_MMAP
 #ifdef SELF_CONTAINED_WRAPPER
 #include <sys/at_ansi.h>
 #include <sys/kd.h>
@@ -98,12 +96,6 @@ extern struct kd_memloc MapDSC;
 extern int mmapFd;
 #endif
 #endif
-#ifndef NO_MMAP
-#include <sys/mman.h>
-#ifndef MAP_FAILED
-#define MAP_FAILED ((caddr_t)-1)
-#endif
-#endif
 #if !defined(ISC)
 #include <stdlib.h>
 #endif
@@ -113,6 +105,13 @@ extern int mmapFd;
 #define DONT_DEFINE_WRAPPERS
 #include "xf86_ansic.h"
 
+#if HAVE_MMAP
+#include <sys/mman.h>
+#ifndef MAP_FAILED
+#define MAP_FAILED ((caddr_t)-1)
+#endif
+#endif
+
 #ifndef SELF_CONTAINED_WRAPPER
 #include "xf86.h"
 #include "xf86Priv.h"
@@ -185,7 +184,7 @@ typedef struct dirent DIRENTRY;
 #ifdef XNO_SYSCONF
 #undef _SC_PAGESIZE
 #endif
-#ifdef HAVE_SYSV_IPC
+#if HAVE_SYSV_IPC
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #endif
@@ -391,10 +390,10 @@ xf86bzero(void* s, unsigned int n)
   
 #ifdef HAVE_VSSCANF
 int
-xf86sscanf(char *s, const char *format, ...)
+xf86sscanf(const char *s, const char *format, ...)
 #else
 int
-xf86sscanf(char *s, const char *format, char *a0, char *a1, char *a2,
+xf86sscanf(const char *s, const char *format, char *a0, char *a1, char *a2,
 	   char *a3, char *a4, char *a5, char *a6, char *a7, char *a8,
 	   char *a9) /* limit of ten args */
 #endif
@@ -512,14 +511,14 @@ void*
 xf86mmap(void *start, xf86size_t length, int prot,
 	 int flags, int fd, xf86size_t /* off_t */ offset)
 {
-#ifndef NO_MMAP
+#if HAVE_MMAP
     int p=0, f=0;
     void *rc;
 
     if (flags & XF86_MAP_FIXED)		f |= MAP_FIXED;
     if (flags & XF86_MAP_SHARED)	f |= MAP_SHARED;
     if (flags & XF86_MAP_PRIVATE)	f |= MAP_PRIVATE;
-#if defined(__AMD64__) && defined(linux)
+#if (defined(__amd64__) || defined(__x86_64__)) && defined(linux)
     if (flags & XF86_MAP_32BIT)	        f |= MAP_32BIT;
 #endif
     if (prot  & XF86_PROT_EXEC)		p |= PROT_EXEC;
@@ -568,7 +567,7 @@ xf86mmap(void *start, xf86size_t length, int prot,
 int
 xf86munmap(void *start, xf86size_t length)
 {
-#ifndef NO_MMAP
+#if HAVE_MMAP
     int rc = munmap(start,(size_t)length);
 
     xf86errno = xf86GetErrno();
@@ -908,7 +907,7 @@ xf86ftell(XF86FILE* f)
 	return ftell(fp->filehnd);
 }
 
-#define mapnum(e) case (xf86_##e): err = e; break;
+#define mapnum(e) case (xf86_##e): err = e; break
 
 char*
 xf86strerror(int n)
@@ -1901,12 +1900,12 @@ xf86GetErrno ()
 
 
 
-#ifdef HAVE_SYSV_IPC
+#if HAVE_SYSV_IPC
 
 int
 xf86shmget(xf86key_t key, int size, int xf86shmflg)
 {
-    int shmflg;
+    int shmflg, status;
 
     /* This copies the permissions (SHM_R, SHM_W for u, g, o). */
     shmflg = xf86shmflg & 0777;
@@ -1916,13 +1915,16 @@ xf86shmget(xf86key_t key, int size, int xf86shmflg)
     if (xf86shmflg & XF86IPC_CREAT) shmflg |= IPC_CREAT;
     if (xf86shmflg & XF86IPC_EXCL) shmflg |= IPC_EXCL;
     if (xf86shmflg & XF86IPC_NOWAIT) shmflg |= IPC_NOWAIT;
-    return shmget((key_t) key, size, shmflg);
+    status = shmget((key_t) key, size, shmflg);
+    xf86errno = xf86GetErrno();
+    return status;
 }
 
 char *
 xf86shmat(int id, char *addr, int xf86shmflg)
 {
     int shmflg = 0;
+    char * status;
 
 #ifdef SHM_RDONLY
     if (xf86shmflg & XF86SHM_RDONLY) shmflg |= SHM_RDONLY;
@@ -1934,32 +1936,62 @@ xf86shmat(int id, char *addr, int xf86shmflg)
     if (xf86shmflg & XF86SHM_REMAP)  shmflg |= SHM_REMAP;
 #endif
 
-    return shmat(id,addr,shmflg);
+    status = shmat(id,addr,shmflg);
+    xf86errno = xf86GetErrno();
+    return status;
 }
 
 int
 xf86shmdt(char *addr)
 {
-    return shmdt(addr);
+    int status = shmdt(addr);
+
+    xf86errno = xf86GetErrno();
+    return status;
 }
 
-/*
- * for now only implement the rmid command.
- */
 int
-xf86shmctl(int id, int xf86cmd, pointer buf)
+xf86shmctl(int id, int xf86cmd, struct xf86shmid_ds * buf)
 {
-    int cmd;
+    struct shmid_ds ds, *dsp;
+    int cmd, status;
 
     switch (xf86cmd) {
     case XF86IPC_RMID:
 	cmd = IPC_RMID;
+	dsp = NULL;
+	break;
+    case XF86IPC_SET:
+	cmd = IPC_SET;
+	dsp = &ds;
+	(void) memset(&ds, 0, sizeof(ds));
+	ds.shm_perm.uid = buf->shm_perm.uid;
+	ds.shm_perm.gid = buf->shm_perm.gid;
+	ds.shm_perm.cuid = buf->shm_perm.cuid;
+	ds.shm_perm.cgid = buf->shm_perm.cgid;
+	ds.shm_perm.mode = buf->shm_perm.mode;
+	ds.shm_segsz = buf->shm_segsz;
+	break;
+    case XF86IPC_STAT:
+	cmd = IPC_STAT;
+	dsp = &ds;
 	break;
     default:
-	return 0;
+	return -1;
     }
     
-    return shmctl(id, cmd, buf);
+    status = shmctl(id, cmd, dsp);
+    xf86errno = xf86GetErrno();
+    if ((status == 0) && (cmd == IPC_STAT)) {
+	(void) memset(buf, 0, sizeof(*buf));
+	buf->shm_perm.uid = ds.shm_perm.uid;
+	buf->shm_perm.gid = ds.shm_perm.gid;
+	buf->shm_perm.cuid = ds.shm_perm.cuid;
+	buf->shm_perm.cgid = ds.shm_perm.cgid;
+	buf->shm_perm.mode = ds.shm_perm.mode;
+	buf->shm_segsz = ds.shm_segsz;
+    }
+    return status;
 }
 #else
 
@@ -1977,7 +2009,7 @@ xf86shmat(int id, char *addr, int xf86shmflg)
 }
 
 int
-xf86shmctl(int id, int xf86cmd, pointer buf)
+xf86shmctl(int id, int xf86cmd, struct xf86shmid_ds * buf)
 {
     return -1;
 }
@@ -2034,6 +2066,8 @@ int
 xf86setjmp1(xf86jmp_buf env, int arg2)
 {
     FatalError("setjmp: type 1 called instead of type %d", xf86getjmptype());
+    /* NOTREACHED */
+    return 0;
 }
 
 #endif  /* HAS_GLIBC_SIGSETJMP */
@@ -2049,5 +2083,7 @@ xf86setjmperror(xf86jmp_buf env)
 {
     FatalError("setjmp: don't know how to handle setjmp() type %d",
 	       xf86getjmptype());
+    /* NOTREACHED */
+    return 0;
 }
 
