@@ -1,10 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/xf4bpp/vgaBitBlt.c,v 1.6 2003/11/17 22:20:42 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/xf4bpp/vgaBitBlt.c,v 1.7 2005/03/28 02:51:07 dawes Exp $ */
 /* GJA -- span move routines */
 
-
-
-/* $XConsortium: vgaBitBlt.c /main/8 1996/10/27 11:06:39 kaleb $ */
-
 #include "xf4bpp.h"
 #include "OScompiler.h"
 #include "vgaReg.h"
@@ -116,15 +112,11 @@ static void shift_thin_rect(
     WindowPtr, int, int, int, int, int, int, int
 );
 
-static void shift_center(
-    WindowPtr, int, int, int, int, int, int, int
-);
+static void shift_center(WindowPtr, int, int, int, int, int, int, int);
 
-void xf4bppBitBlt(pWin,alu,writeplanes,x0,y0,x1,y1,w,h)
-WindowPtr pWin; /* GJA */
-int alu;
-int writeplanes; /* planes */
-int x0, y0, x1, y1, w, h;
+void
+xf4bppBitBlt(WindowPtr pWin, int alu, int writeplanes, int x0, int y0,
+	     int x1, int y1, int w, int h)
 {
     IOADDRESS REGBASE;
     int plane, bit;
@@ -166,15 +158,7 @@ int x0, y0, x1, y1, w, h;
 /* Copy a span a number of places to the right.
  */
 static void
-shift(pWin,x0,x1,y0,y1,w,h,alu)
-WindowPtr pWin; /* GJA */
-int x0;  /* left edge of source */
-int x1;  /* left edge of target */
-int y0;
-int y1;
-int w; /* length of source, and of target */
-int h;
-int alu;
+shift(WindowPtr pWin, int x0, int x1, int y0, int y1, int w, int h, int alu)
 {
   if ( ((x1 & WMASK) + w) <= WORDSZ ) {
      shift_thin_rect(pWin,x0,x1,y0,y1,w,h,alu);
@@ -199,15 +183,8 @@ int alu;
 
 /* The whole rectangle is so thin that it fits in one byte written */
 static void
-shift_thin_rect(pWin,x0,x1,y0,y1,w,h,alu)
-WindowPtr pWin; /* GJA */
-int x0;  /* left edge of source */
-int x1;  /* left edge of target */
-int y0;
-int y1;
-int w; /* length of source, and of target */
-int h;
-int alu;
+shift_thin_rect(WindowPtr pWin, int x0, int x1, int y0, int y1, int w, int h,
+		int alu)
 {
   int l0 = x0 & WMASK; /* Left edge of source, as bit */
   int l1 = x1 & WMASK; /* Left edge of target, as bit */
@@ -265,15 +242,8 @@ int alu;
 }
 
 static void
-shift_center(pWin,x0,x1,y0,y1,w,h,alu)
-WindowPtr pWin; /* GJA */
-int x0;  /* left edge of source */
-int x1;  /* left edge of target */
-int y0;
-int y1;
-int w; /* length of source, and of target */
-int h;
-int alu;
+shift_center(WindowPtr pWin, int x0, int x1, int y0, int y1, int w, int h,
+	     int alu)
 {
   int l1 = x1 & WMASK; /* Left edge of target, as bit */
   int r1 = (x1 + w) & WMASK; /* Right edge of target, as bit */
@@ -388,16 +358,8 @@ int alu;
 /* Copy a rectangle.
  */
 static void
-aligned_blit(pWin,x0,x1,y0,y1,w,h,alu,planes)
-WindowPtr pWin; /* GJA */
-int x0;  /* left edge of source */
-int x1;  /* left edge of target */
-int y0;
-int y1;
-int w; /* length of source, and of target */
-int h;
-int alu;
-int planes;
+aligned_blit(WindowPtr pWin, int x0, int x1, int y0, int y1, int w, int h,
+	     int alu, int planes)
 {
   IOADDRESS REGBASE =
 	xf86Screens[((DrawablePtr)pWin)->pScreen->myNum]->domainIOBase + 0x300;
@@ -511,14 +473,8 @@ int planes;
 }
 
 static void
-aligned_blit_center(pWin,x0,x1,y0,y1,w,h)
-WindowPtr pWin; /* GJA */
-int x0;  /* left edge of source */
-int x1;  /* left edge of target */
-int y0;
-int y1;
-int w; /* length of source, and of target */
-int h;
+aligned_blit_center(WindowPtr pWin, int x0, int x1, int y0, int y1, int w,
+		    int h)
 {
   int l1 = x1 & WMASK; /* Left edge of target, as bit */
   int r1 = (x1 + w) & WMASK; /* Right edge of target, as bit */
@@ -588,20 +544,16 @@ int h;
 #else	/* PC98_EGC */
 
 static void
-egc_fast_blt (pWin, alu, writeplanes, x0, y0, x1, y1, w, h)
-WindowPtr pWin;	
-const	int alu, writeplanes ;
-register int x0, x1 ;
-int	     y0, y1 ;
-register int w, h ;
+egc_fast_blt(WindowPtr pWin, const int alu, const int writeplanes,
+	     int x0, int y0, int x1, int y1, int w, int h)
 {
-register volatile unsigned char *src ;
-register volatile unsigned char *dst ;
+volatile unsigned char *src ;
+volatile unsigned char *dst ;
 unsigned short *src_x ;
 unsigned short *dst_x ;
 int x_direction, y_interval ;
 int	src_off, dst_off ;
-register int k, i ;
+int k, i ;
 unsigned short ROP_value;
 
 src = (unsigned char *)SCREENADDRESS( pWin, 0, y0);
@@ -720,11 +672,8 @@ return;
 }
 
 void
-xf4bppBitBlt( pWin,alu, writeplanes, x0, y0, x1, y1, w, h )
-WindowPtr pWin; /* GJA */
-int alu;
-int writeplanes; /* planes */
-int x0, y0, x1, y1, w, h;
+xf4bppBitBlt(WindowPtr pWin, int alu, int writeplanes, int x0, int y0,
+	     int x1, int y1, int w, int h)
 {
 	if ( ! xf86Screens[((DrawablePtr)pWin)->pScreen->myNum]->vtSema ) {
 		xf4bppOffBitBlt( pWin, alu, writeplanes,
