@@ -1,4 +1,3 @@
-/* $Xorg: access.c,v 1.5 2001/02/09 02:05:23 xorgcvs Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -45,7 +44,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/programs/Xserver/os/access.c,v 3.54 2004/01/03 17:38:39 herrb Exp $ */
+/* $XFree86: xc/programs/Xserver/os/access.c,v 3.60 2005/10/14 15:17:26 tsi Exp $ */
 
 #ifdef WIN32
 #include <X11/Xwinsock.h>
@@ -55,8 +54,8 @@ SOFTWARE.
 #include <stdlib.h>
 #include <X11/Xtrans.h>
 #include <X11/Xauth.h>
-#include <X.h>
-#include <Xproto.h>
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "misc.h"
 #include "site.h"
 #include <errno.h>
@@ -70,9 +69,9 @@ SOFTWARE.
 #include <sys/ioctl.h>
 #include <ctype.h>
 
-#if defined(TCPCONN) || defined(STREAMSCONN) || defined(ISC) || defined(SCO)
+#if defined(TCPCONN) || defined(STREAMSCONN) || defined(ISC) || defined(__SCO__)
 #include <netinet/in.h>
-#endif /* TCPCONN || STREAMSCONN || ISC || SCO */
+#endif /* TCPCONN || STREAMSCONN || ISC || __SCO__ */
 #ifdef DNETCONN
 #include <netdnet/dn.h>
 #include <netdnet/dnetdb.h>
@@ -118,7 +117,7 @@ SOFTWARE.
 #endif /* hpux */
 
 #ifdef SVR4
-#ifndef SCO
+#ifndef __SCO__
 #include <sys/sockio.h>
 #endif
 #include <sys/stropts.h>
@@ -181,7 +180,7 @@ SOFTWARE.
 
 #ifdef XCSECURITY
 #define _SECURITY_SERVER
-#include "extensions/security.h"
+#include <X11/extensions/security.h>
 #endif
 
 #ifndef PATH_MAX
@@ -287,7 +286,7 @@ AccessUsingXdmcp (void)
 }
 
 
-#if ((defined(SVR4) && !defined(DGUX) && !defined(SCO325) && !defined(sun) && !defined(NCR)) || defined(ISC)) && defined(SIOCGIFCONF) && !defined(USE_SIOCGLIFCONF)
+#if ((defined(SVR4) && !defined(DGUX) && !defined(__SCO__) && !defined(sun) && !defined(NCR)) || defined(ISC)) && !defined(__sgi) && defined(SIOCGIFCONF) && !defined(USE_SIOCGLIFCONF)
 
 /* Deal with different SIOCGIFCONF ioctl semantics on these OSs */
 
@@ -336,9 +335,9 @@ ifioctl (int fd, int cmd, char *arg)
 #endif
     return(ret);
 }
-#else /* Case DGUX, sun, SCO325 NCR and others  */
+#else /* Case DGUX, sun, __SCO__ NCR and others  */
 #define ifioctl ioctl
-#endif /* ((SVR4 && !DGUX !sun !SCO325 !NCR) || ISC) && SIOCGIFCONF */
+#endif /* ((SVR4 && !DGUX !sun !__SCO__ !NCR) || ISC) && SIOCGIFCONF */
 
 /*
  * DefineSelf (fd):
@@ -374,7 +373,7 @@ DefineSelf (int fd)
     struct in_ifaddr ifaddr;
     struct strioctl str;
     unsigned char *addr;
-    register HOST *host;
+    HOST *host;
     int	family, len;
 
     if ((fd = open ("/dev/ip", O_RDWR, 0 )) < 0)
@@ -474,7 +473,6 @@ DefineSelf (int fd)
 	{
 	    host->family = FamilyLocalHost;
 	    host->len = 0;
-	    acopy("", host->addr, 0);
 	    host->next = selfhosts;
 	    selfhosts = host;
 	}
@@ -490,14 +488,14 @@ DefineSelf (int fd)
 #if !defined(TCPCONN) && !defined(STREAMSCONN) && !defined(UNIXCONN) && !defined(MNX_TCPCONN)
     return;
 #else
-    register int n;
+    int n;
     int	len;
     caddr_t	addr;
     int		family;
-    register HOST	*host;
+    HOST	*host;
 
     struct utsname name;
-    register struct hostent  *hp;
+    struct hostent  *hp;
 
     union {
 	struct  sockaddr   sa;
@@ -610,7 +608,6 @@ DefineLocalHost:
 	{
 	    host->family = FamilyLocalHost;
 	    host->len = 0;
-	    acopy("", host->addr, 0);
 	    host->next = selfhosts;
 	    selfhosts = host;
 	}
@@ -667,13 +664,13 @@ DefineSelf (int fd)
     void *		bufptr = buf;   
 #ifdef USE_SIOCGLIFCONF
     struct lifconf	ifc;
-    register struct lifreq *ifr;
+    struct lifreq *ifr;
 #ifdef SIOCGLIFNUM
     struct lifnum	ifn;
 #endif
 #else
     struct ifconf	ifc;
-    register struct ifreq *ifr;
+    struct ifreq *ifr;
 #endif 
 #else 
     struct ifaddrs *	ifap, *ifr;
@@ -681,7 +678,7 @@ DefineSelf (int fd)
     int 		len;
     unsigned char *	addr;
     int 		family;
-    register HOST 	*host;
+    HOST 	*host;
     
 #ifdef DNETCONN
     struct dn_naddr *dnaddr = getnodeadd();
@@ -1015,7 +1012,6 @@ DefineSelf (int fd)
 	{
 	    host->family = FamilyLocalHost;
 	    host->len = 0;
-	    acopy("", host->addr, 0);
 	    host->next = selfhosts;
 	    selfhosts = host;
 	}
@@ -1030,7 +1026,7 @@ AugmentSelf(pointer from, int len)
 {
     int family;
     pointer addr;
-    register HOST *host;
+    HOST *host;
 
     family = ConvertAddr(from, &len, (pointer *)&addr);
     if (family == -1 || family == FamilyLocal)
@@ -1068,7 +1064,7 @@ AddLocalHosts (void)
 void
 ResetHosts (char *display)
 {
-    register HOST	*host;
+    HOST	*host;
     char                lhostname[120], ohostname[120];
     char 		*hostname = ohostname;
     char		fname[PATH_MAX + 1];
@@ -1246,7 +1242,7 @@ ResetHosts (char *display)
 #ifdef XTHREADS_NEEDS_BYNAMEPARAMS
 	    _Xgethostbynameparams hparams;
 #endif
-	    register struct hostent *hp;
+	    struct hostent *hp;
 
     	    /* host name */
     	    if ((family == FamilyInternet &&
@@ -1283,7 +1279,7 @@ Bool LocalClient(ClientPtr client)
     int    		alen, family, notused;
     Xtransaddr		*from = NULL;
     pointer		addr;
-    register HOST	*host;
+    HOST	*host;
 
 #ifdef XCSECURITY
     /* untrusted clients can't change host access */
@@ -1457,7 +1453,7 @@ NewHost (int		family,
 	 int		len,
 	 int		addingLocalHosts)
 {
-    register HOST *host;
+    HOST *host;
 
     for (host = validhosts; host; host = host->next)
     {
@@ -1493,7 +1489,7 @@ RemoveHost (
     pointer             pAddr)
 {
     int			len;
-    register HOST	*host, **prev;
+    HOST	*host, **prev;
 
     if (!AuthorizedClient(client))
 	return(BadAccess);
@@ -1550,9 +1546,9 @@ GetHosts (
     BOOL		*pEnabled)
 {
     int			len;
-    register int 	n = 0;
-    register unsigned char *ptr;
-    register HOST	*host;
+    int 	n = 0;
+    unsigned char *ptr;
+    HOST	*host;
     int			nHosts = 0;
 
     *pEnabled = AccessEnabled ? EnableAccess : DisableAccess;
@@ -1598,7 +1594,7 @@ CheckAddr (
 
     switch (family)
     {
-#if defined(TCPCONN) || defined(STREAMSCONN) || defined(AMTCPCONN) || defined(MNX_TCPCONN)
+#if defined(TCPCONN) || defined(STREAMSCONN) || defined(MNX_TCPCONN)
       case FamilyInternet:
 	if (length == sizeof (struct in_addr))
 	    len = length;
@@ -1640,12 +1636,12 @@ CheckAddr (
 
 int
 InvalidHost (
-    register struct sockaddr	*saddr,
+    struct sockaddr	*saddr,
     int				len)
 {
     int 			family;
     pointer			addr;
-    register HOST 		*selfhost, *host;
+    HOST 		*selfhost, *host;
 
     if (!AccessEnabled)   /* just let them in */
         return(0);    
@@ -1683,7 +1679,7 @@ InvalidHost (
 
 static int
 ConvertAddr (
-    register struct sockaddr	*saddr,
+    struct sockaddr	*saddr,
     int				*len,
     pointer			*addr)
 {
