@@ -48,9 +48,9 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 OR PERFORMANCE OF THIS SOFTWARE.
 
 */
-/* $XFree86: xc/programs/Xserver/os/utils.c,v 3.102 2005/02/03 02:01:14 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/os/utils.c,v 3.110 2006/03/21 03:56:27 dawes Exp $ */
 /*
- * Copyright (c) 1996-2005 by The XFree86 Project, Inc.
+ * Copyright (c) 1996-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -104,10 +104,10 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #if defined(WIN32) && !defined(__CYGWIN__)
 #include <X11/Xwinsock.h>
 #endif
-#include "Xos.h"
+#include <X11/Xos.h>
 #include <stdio.h>
 #include "misc.h"
-#include "X.h"
+#include <X11/X.h>
 #include <X11/Xtrans.h>
 #include "input.h"
 #include "dixfont.h"
@@ -154,11 +154,16 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #endif
 
 #ifdef XKB
-#include "XKBsrv.h"
+#include <X11/extensions/XKBsrv.h>
 #endif
+
 #ifdef XCSECURITY
 #define _SECURITY_SERVER
-#include "security.h"
+#include <X11/extensions/security.h>
+#endif
+
+#ifdef XPRINT
+#include "DiPrint.h"
 #endif
 
 #ifdef RENDER
@@ -173,8 +178,8 @@ OR PERFORMANCE OF THIS SOFTWARE.
 Bool CoreDump;
 Bool noTestExtensions;
 
-Bool noPanoramiXExtension = TRUE;
 #ifdef PANORAMIX
+Bool noPanoramiXExtension = TRUE;
 Bool PanoramiXVisibilityNotifySent = FALSE;
 Bool PanoramiXMapped = FALSE;
 Bool PanoramiXWindowExposureSent = FALSE;
@@ -473,13 +478,44 @@ GiveUp(int sig)
     errno = olderrno;
 }
 
+/*ARGSUSED*/
+SIGVAL
+AbortServer(int sig)
+{
+    static Bool beenHere = FALSE;
+
+#if defined(SYSV) && defined(X_NOT_POSIX)
+    if (sig)
+	OsSignal(sig, SIG_IGN);
+#endif
+    /*
+     * Once entering here, ignore SIGINT and SIGTERM so that we don't
+     * re-enter.
+     */
+    OsSignal(SIGINT, SIG_IGN);
+    OsSignal(SIGTERM, SIG_IGN);
+    if (!beenHere) {
+	beenHere = TRUE;
+	if (sig == SIGINT)
+	    ErrorF("X server terminated by an interrupt signal.\n");
+	OsCleanup(TRUE);
+	AbortDDX();
+    } else {
+	ErrorF("Re-entering AbortServer.  Aborting without further cleanup.\n");
+    }
+    fflush(stderr);
+    if (CoreDump)
+	abort();
+    exit (1);
+}
+
 #ifndef DDXTIME
 CARD32
 GetTimeInMillis(void)
 {
     struct timeval  tp;
-    register CARD32 val;
-    register INT32 diff;
+    CARD32 val;
+    INT32 diff;
     static CARD32 oldval = 0;
     static CARD32 time = 0;
 
@@ -1243,19 +1279,19 @@ set_font_authorizations(char **authorizations, int *authlen, pointer client)
 void * 
 Xalloc(unsigned long amount)
 {
-    register pointer  ptr;
-	
-    if ((long)amount <= 0)
-	return NULL;
+    pointer ptr;
 
     /* aligned extra on long word boundary */
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
+
+    if ((long)amount <= 0)
+	return NULL;
 #ifdef MEMBUG
     if (!Must_have_memory && Memory_fail &&
 	((random() % MEM_FAIL_SCALE) < Memory_fail))
 	return NULL;
 #endif
-    if ((ptr = (pointer)malloc(amount))) {
+    if ((ptr = malloc(amount))) {
 	return ptr;
     }
     if (Must_have_memory)
@@ -1271,14 +1307,18 @@ Xalloc(unsigned long amount)
 void *
 XNFalloc(unsigned long amount)
 {
-    register pointer ptr;
+    pointer ptr;
 
-    if ((long)amount <= 0)
-        return NULL;
+    if (amount == 0)
+	return NULL;
 
     /* aligned extra on long word boundary */
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
-    ptr = (pointer)malloc(amount);
+
+    if ((long)amount <= 0)
+        FatalError("Bad request for memory");
+
+    ptr = malloc(amount);
     if (!ptr)
         FatalError("Out of memory");
 
@@ -1292,11 +1332,11 @@ XNFalloc(unsigned long amount)
 void *
 Xcalloc(unsigned long amount)
 {
-    unsigned long   *ret;
+    pointer ret;
 
-    ret = Xalloc (amount);
+    ret = Xalloc(amount);
     if (ret)
-	bzero ((void *) ret, (int) amount);
+	bzero (ret, (int) amount);
     return ret;
 }
 
@@ -1307,13 +1347,17 @@ Xcalloc(unsigned long amount)
 void *
 XNFcalloc(unsigned long amount)
 {
-    unsigned long   *ret;
+    pointer ret;
 
-    ret = Xalloc (amount);
-    if (ret)
-	bzero ((char *) ret, (int) amount);
-    else if ((long)amount > 0)
+    if (amount == 0)
+	return NULL;
+
+    ret = Xalloc(amount);
+    if (!ret)
         FatalError("Out of memory");
+
+    bzero (ret, (int) amount);
+
     return ret;
 }
 
@@ -1336,10 +1380,12 @@ Xrealloc(pointer ptr, unsigned long amount)
 	return NULL;
     }
     amount = (amount + (sizeof(long) - 1)) & ~(sizeof(long) - 1);
+    if ((long)amount <= 0)
+	return NULL;
     if (ptr)
-        ptr = (pointer)realloc((char *)ptr, amount);
+        ptr = realloc(ptr, amount);
     else
-	ptr = (pointer)malloc(amount);
+	ptr = malloc(amount);
     if (ptr)
         return ptr;
     if (Must_have_memory)
@@ -1355,12 +1401,12 @@ Xrealloc(pointer ptr, unsigned long amount)
 void *
 XNFrealloc(pointer ptr, unsigned long amount)
 {
-    if (( ptr = (pointer)Xrealloc( ptr, amount ) ) == NULL)
+    if ((ptr = Xrealloc(ptr, amount)) == NULL)
     {
-	if ((long)amount > 0)
+	if (amount != 0)
             FatalError( "Out of memory" );
     }
-    return ((unsigned long *)ptr);
+    return ptr;
 }
 
 /*****************
@@ -2173,3 +2219,18 @@ CheckUserAuthorization(void)
     }
 #endif
 }
+
+int
+getArgc()
+{
+    return argcGlobal;
+}
+
+const char *
+getArgv(int i)
+{
+    if (i >= 0 && i < argcGlobal)
+	return argvGlobal[i];
+    else
+	return NULL;
+}
