@@ -1,4 +1,3 @@
-/* $Xorg: xkbEvents.c,v 1.3 2000/08/17 19:53:47 cpqbld Exp $ */
 /************************************************************
 Copyright (c) 1993 by Silicon Graphics Computer Systems, Inc.
 
@@ -24,17 +23,17 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ********************************************************/
-/* $XFree86: xc/programs/Xserver/xkb/xkbEvents.c,v 3.12 2003/11/17 22:20:46 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/xkb/xkbEvents.c,v 3.14 2005/10/14 15:17:28 tsi Exp $ */
 
 #include <stdio.h>
 #define NEED_EVENTS 1
 #include <X11/X.h>
 #include <X11/Xproto.h>
 #include <X11/keysym.h>
-#include "XI.h"
+#include <X11/extensions/XI.h>
 #include "inputstr.h"
 #include "windowstr.h"
-#include "XKBsrv.h"
+#include <X11/extensions/XKBsrv.h>
 #include "xkb.h"
 
 /***====================================================================***/
@@ -42,7 +41,7 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 void
 XkbSendNewKeyboardNotify(DeviceIntPtr kbd,xkbNewKeyboardNotify *pNKN)
 {		
-register int	i;
+int	i;
 Time 		time;
 CARD16		changed;
 
@@ -63,7 +62,7 @@ CARD16		changed;
 		pNKN->time = time;
 		pNKN->changed = changed;
 		if ( clients[i]->swapped ) {
-		    register int n;
+		    int n;
 		    swaps(&pNKN->sequenceNumber,n);
 		    swapl(&pNKN->time,n);
 		    swaps(&pNKN->changed,n);
@@ -103,7 +102,7 @@ XkbSrvInfoPtr	xkbi;
 XkbStatePtr	state;
 XkbInterestPtr	interest;
 Time 		time;
-register CARD16	changed,bState;
+CARD16	changed,bState;
 
     interest = kbd->xkb_interest;
     if (!interest)
@@ -142,7 +141,7 @@ register CARD16	changed,bState;
 	    pSN->changed = changed;
 	    pSN->ptrBtnState = bState;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pSN->sequenceNumber,n);
 		swapl(&pSN->time,n);
 		swaps(&pSN->changed,n);
@@ -188,7 +187,7 @@ CARD16		changed;
 	    pMN->sequenceNumber = clients[i]->sequence;
 	    pMN->changed = changed;
 	    if ( clients[i]->swapped ) {
-		register int n;
+		int n;
 		swaps(&pMN->sequenceNumber,n);
 		swapl(&pMN->time,n);
 		swaps(&pMN->changed,n);
@@ -311,7 +310,7 @@ Time 		 	time = 0;
 	    pCN->sequenceNumber = interest->client->sequence;
 	    pCN->time = time;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pCN->sequenceNumber,n);
 		swapl(&pCN->changedControls,n);
 		swapl(&pCN->enabledControls,n);
@@ -360,7 +359,7 @@ CARD32		state,changed;
 	    pEv->changed = changed;
 	    pEv->state = state;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 		swapl(&pEv->changed,n);
@@ -443,7 +442,7 @@ XID		winID = 0;
 	    bn.name = name;
 	    bn.window=  winID;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&bn.sequenceNumber,n);
 		swapl(&bn.time,n);
 		swaps(&bn.pitch,n);
@@ -490,7 +489,7 @@ CARD16		sk_delay,db_delay;
 	    pEv->slowKeysDelay = sk_delay;
 	    pEv->debounceDelay = db_delay;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 		swaps(&pEv->slowKeysDelay,n);
@@ -538,7 +537,7 @@ CARD32		changedIndicators;
 	    pEv->changedIndicators = changedIndicators;
 	    pEv->changedVirtualMods= changedVirtualMods;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 		swaps(&pEv->changed,n);
@@ -586,7 +585,7 @@ CARD16		firstSI = 0, nSI = 0, nTotalSI = 0;
 	    pEv->nSI = nSI;
 	    pEv->nTotalSI = nTotalSI;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 		swaps(&pEv->firstSI,n);
@@ -632,7 +631,7 @@ Time 		 time = 0;
 	    pEv->sequenceNumber = interest->client->sequence;
 	    pEv->time = time;
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 	    }
@@ -692,7 +691,7 @@ CARD16		 reason, supported = 0;
 		    continue;
 	    }
 	    if ( interest->client->swapped ) {
-		register int n;
+		int n;
 		swaps(&pEv->sequenceNumber,n);
 		swapl(&pEv->time,n);
 		swapl(&pEv->ledsDefined,n);
@@ -827,7 +826,7 @@ XkbSrvInfoPtr	xkbi;
 	}
 	if ((pXDev->grab != NullGrab) && pXDev->fromPassiveGrab &&
 	    ((xE[0].u.u.type==KeyPress)||(xE[0].u.u.type==KeyRelease))) {
-	    register unsigned state,flags;
+	    unsigned state,flags;
 
 	    flags= pClient->xkbClientFlags;
 	    state= xkbi->state.compat_grab_mods;
@@ -865,7 +864,7 @@ XkbSrvInfoPtr	xkbi;
         }
     }
     else {
-	register CARD8 	type;
+	CARD8 	type;
 
 	for (i=0;i<nEvents;i++) {
 	    type= xE[i].u.u.type;
