@@ -1,4 +1,3 @@
-/* $Xorg: xkbInit.c,v 1.3 2000/08/17 19:53:47 cpqbld Exp $ */
 /************************************************************
 Copyright (c) 1993 by Silicon Graphics Computer Systems, Inc.
 
@@ -24,7 +23,7 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ********************************************************/
-/* $XFree86: xc/programs/Xserver/xkb/xkbInit.c,v 3.33 2003/12/22 17:48:12 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/xkb/xkbInit.c,v 3.36 2005/10/14 15:17:28 tsi Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,8 +40,8 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "opaque.h"
 #include "property.h"
 #define	XKBSRV_NEED_FILE_FUNCS
-#include "XKBsrv.h"
-#include "XKBgeom.h"
+#include <X11/extensions/XKBsrv.h>
+#include <X11/extensions/XKBgeom.h>
 #include <X11/extensions/XKMformat.h>
 #include <X11/extensions/XKBfile.h>
 #include "xkb.h"
@@ -310,7 +309,7 @@ XkbInitRadioGroups(XkbSrvInfoPtr xkbi,SrvXkmInfo *file)
 static Status
 XkbInitCompatStructs(XkbDescPtr xkb,SrvXkmInfo *file)
 {
-register int 	i;
+int 	i;
 XkbCompatMapPtr	compat;
 
     if (file->xkbinfo.defined&XkmCompatMapMask)
@@ -624,6 +623,7 @@ XkbRF_VarDefsRec	defs;
     pMods= pModsIn;
     bzero(&defs,sizeof(XkbRF_VarDefsRec));
     bzero(&cfgNames,sizeof(XkbComponentNamesRec));
+    bzero(&finfo, sizeof(finfo));
     rules= XkbGetRulesDflts(&defs);
     config= XkbDDXPreloadConfig(&rules,&defs,&cfgNames,dev);
 
